# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

from openlp.core.lib import translate, build_icon
from openlp.core.lib.theme import HorizontalType, BackgroundType, \
    BackgroundGradientType
from openlp.core.lib.ui import UiStrings, add_welcome_page, \
    create_valign_selection_widgets


class Ui_ThemeWizard(object):
    def setupUi(self, themeWizard):
        themeWizard.setObjectName(u'OpenLP.ThemeWizard')
        themeWizard.setModal(True)
        themeWizard.setWizardStyle(QtGui.QWizard.ModernStyle)
        themeWizard.setOptions(QtGui.QWizard.IndependentPages |
            QtGui.QWizard.NoBackButtonOnStartPage |
            QtGui.QWizard.HaveCustomButton1)
        themeWizard.setWindowIcon(build_icon(u':/icon/openlp-logo.svg'))
        self.spacer = QtGui.QSpacerItem(10, 0,
            QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        # Welcome Page
        add_welcome_page(themeWizard, u':/wizards/wizard_createtheme.bmp')
        # Background Page
        self.backgroundPage = QtGui.QWizardPage()
        self.backgroundPage.setObjectName(u'BackgroundPage')
        self.backgroundLayout = QtGui.QVBoxLayout(self.backgroundPage)
        self.backgroundLayout.setObjectName(u'BackgroundLayout')
        self.backgroundTypeLayout = QtGui.QFormLayout()
        self.backgroundTypeLayout.setObjectName(u'BackgroundTypeLayout')
        self.backgroundLabel = QtGui.QLabel(self.backgroundPage)
        self.backgroundLabel.setObjectName(u'BackgroundLabel')
        self.backgroundComboBox = QtGui.QComboBox(self.backgroundPage)
        self.backgroundComboBox.addItems([u'', u'', u'', u''])
        self.backgroundComboBox.setObjectName(u'BackgroundComboBox')
        self.backgroundTypeLayout.addRow(self.backgroundLabel,
            self.backgroundComboBox)
        self.backgroundTypeLayout.setItem(1, QtGui.QFormLayout.LabelRole,
            self.spacer)
        self.backgroundLayout.addLayout(self.backgroundTypeLayout)
        self.backgroundStack = QtGui.QStackedLayout()
        self.backgroundStack.setObjectName(u'BackgroundStack')
        self.colorWidget = QtGui.QWidget(self.backgroundPage)
        self.colorWidget.setObjectName(u'ColorWidget')
        self.colorLayout = QtGui.QFormLayout(self.colorWidget)
        self.colorLayout.setMargin(0)
        self.colorLayout.setObjectName(u'ColorLayout')
        self.colorLabel = QtGui.QLabel(self.colorWidget)
        self.colorLabel.setObjectName(u'ColorLabel')
        self.colorButton = QtGui.QPushButton(self.colorWidget)
        self.colorButton.setObjectName(u'ColorButton')
        self.colorLayout.addRow(self.colorLabel, self.colorButton)
        self.colorLayout.setItem(1, QtGui.QFormLayout.LabelRole, self.spacer)
        self.backgroundStack.addWidget(self.colorWidget)
        self.gradientWidget = QtGui.QWidget(self.backgroundPage)
        self.gradientWidget.setObjectName(u'GradientWidget')
        self.gradientLayout = QtGui.QFormLayout(self.gradientWidget)
        self.gradientLayout.setMargin(0)
        self.gradientLayout.setObjectName(u'GradientLayout')
        self.gradientStartLabel = QtGui.QLabel(self.gradientWidget)
        self.gradientStartLabel.setObjectName(u'GradientStartLabel')
        self.gradientStartButton = QtGui.QPushButton(self.gradientWidget)
        self.gradientStartButton.setObjectName(u'GradientStartButton')
        self.gradientLayout.addRow(self.gradientStartLabel,
            self.gradientStartButton)
        self.gradientEndLabel = QtGui.QLabel(self.gradientWidget)
        self.gradientEndLabel.setObjectName(u'GradientEndLabel')
        self.gradientEndButton = QtGui.QPushButton(self.gradientWidget)
        self.gradientEndButton.setObjectName(u'GradientEndButton')
        self.gradientLayout.addRow(self.gradientEndLabel,
            self.gradientEndButton)
        self.gradientTypeLabel = QtGui.QLabel(self.gradientWidget)
        self.gradientTypeLabel.setObjectName(u'GradientTypeLabel')
        self.gradientComboBox = QtGui.QComboBox(self.gradientWidget)
        self.gradientComboBox.setObjectName(u'GradientComboBox')
        self.gradientComboBox.addItems([u'', u'', u'', u'', u''])
        self.gradientLayout.addRow(self.gradientTypeLabel,
            self.gradientComboBox)
        self.gradientLayout.setItem(3, QtGui.QFormLayout.LabelRole, self.spacer)
        self.backgroundStack.addWidget(self.gradientWidget)
        self.imageWidget = QtGui.QWidget(self.backgroundPage)
        self.imageWidget.setObjectName(u'ImageWidget')
        self.imageLayout = QtGui.QFormLayout(self.imageWidget)
        self.imageLayout.setMargin(0)
        self.imageLayout.setObjectName(u'ImageLayout')
        self.imageColorLabel = QtGui.QLabel(self.colorWidget)
        self.imageColorLabel.setObjectName(u'ImageColorLabel')
        self.imageColorButton = QtGui.QPushButton(self.colorWidget)
        self.imageColorButton.setObjectName(u'ImageColorButton')
        self.imageLayout.addRow(self.imageColorLabel, self.imageColorButton)
        self.imageLabel = QtGui.QLabel(self.imageWidget)
        self.imageLabel.setObjectName(u'ImageLabel')
        self.imageFileLayout = QtGui.QHBoxLayout()
        self.imageFileLayout.setObjectName(u'ImageFileLayout')
        self.imageFileEdit = QtGui.QLineEdit(self.imageWidget)
        self.imageFileEdit.setObjectName(u'ImageFileEdit')
        self.imageFileLayout.addWidget(self.imageFileEdit)
        self.imageBrowseButton = QtGui.QToolButton(self.imageWidget)
        self.imageBrowseButton.setObjectName(u'ImageBrowseButton')
        self.imageBrowseButton.setIcon(
            build_icon(u':/general/general_open.png'))
        self.imageFileLayout.addWidget(self.imageBrowseButton)
        self.imageLayout.addRow(self.imageLabel, self.imageFileLayout)
        self.imageLayout.setItem(2, QtGui.QFormLayout.LabelRole, self.spacer)
        self.backgroundStack.addWidget(self.imageWidget)
        self.transparentWidget = QtGui.QWidget(self.backgroundPage)
        self.transparentWidget.setObjectName(u'TransparentWidget')
        self.transparentLayout = QtGui.QFormLayout(self.transparentWidget)
        self.transparentLayout.setMargin(0)
        self.transparentLayout.setObjectName(u'TransparentLayout')
        self.backgroundStack.addWidget(self.transparentWidget)
        self.backgroundLayout.addLayout(self.backgroundStack)
        themeWizard.addPage(self.backgroundPage)
        # Main Area Page
        self.mainAreaPage = QtGui.QWizardPage()
        self.mainAreaPage.setObjectName(u'MainAreaPage')
        self.mainAreaLayout = QtGui.QFormLayout(self.mainAreaPage)
        self.mainAreaLayout.setObjectName(u'MainAreaLayout')
        self.mainFontLabel = QtGui.QLabel(self.mainAreaPage)
        self.mainFontLabel.setObjectName(u'MainFontLabel')
        self.mainFontComboBox = QtGui.QFontComboBox(self.mainAreaPage)
        self.mainFontComboBox.setObjectName(u'MainFontComboBox')
        self.mainAreaLayout.addRow(self.mainFontLabel, self.mainFontComboBox)
        self.mainColorLabel = QtGui.QLabel(self.mainAreaPage)
        self.mainColorLabel.setObjectName(u'MainColorLabel')
        self.mainPropertiesLayout = QtGui.QHBoxLayout()
        self.mainPropertiesLayout.setObjectName(u'MainPropertiesLayout')
        self.mainColorButton = QtGui.QPushButton(self.mainAreaPage)
        self.mainColorButton.setObjectName(u'MainColorButton')
        self.mainPropertiesLayout.addWidget(self.mainColorButton)
        self.mainPropertiesLayout.addSpacing(20)
        self.mainBoldCheckBox = QtGui.QCheckBox(self.mainAreaPage)
        self.mainBoldCheckBox.setObjectName(u'MainBoldCheckBox')
        self.mainPropertiesLayout.addWidget(self.mainBoldCheckBox)
        self.mainPropertiesLayout.addSpacing(20)
        self.mainItalicsCheckBox = QtGui.QCheckBox(self.mainAreaPage)
        self.mainItalicsCheckBox.setObjectName(u'MainItalicsCheckBox')
        self.mainPropertiesLayout.addWidget(self.mainItalicsCheckBox)
        self.mainAreaLayout.addRow(self.mainColorLabel,
            self.mainPropertiesLayout)
        self.mainSizeLabel = QtGui.QLabel(self.mainAreaPage)
        self.mainSizeLabel.setObjectName(u'MainSizeLabel')
        self.mainSizeLayout = QtGui.QHBoxLayout()
        self.mainSizeLayout.setObjectName(u'MainSizeLayout')
        self.mainSizeSpinBox = QtGui.QSpinBox(self.mainAreaPage)
        self.mainSizeSpinBox.setMaximum(999)
        self.mainSizeSpinBox.setValue(16)
        self.mainSizeSpinBox.setObjectName(u'MainSizeSpinBox')
        self.mainSizeLayout.addWidget(self.mainSizeSpinBox)
        self.mainLineCountLabel = QtGui.QLabel(self.mainAreaPage)
        self.mainLineCountLabel.setObjectName(u'MainLineCountLabel')
        self.mainSizeLayout.addWidget(self.mainLineCountLabel)
        self.mainAreaLayout.addRow(self.mainSizeLabel, self.mainSizeLayout)
        self.lineSpacingLabel = QtGui.QLabel(self.mainAreaPage)
        self.lineSpacingLabel.setObjectName(u'LineSpacingLabel')
        self.lineSpacingSpinBox = QtGui.QSpinBox(self.mainAreaPage)
        self.lineSpacingSpinBox.setMinimum(-250)
        self.lineSpacingSpinBox.setMaximum(250)
        self.lineSpacingSpinBox.setObjectName(u'LineSpacingSpinBox')
        self.mainAreaLayout.addRow(self.lineSpacingLabel,
            self.lineSpacingSpinBox)
        self.outlineCheckBox = QtGui.QCheckBox(self.mainAreaPage)
        self.outlineCheckBox.setObjectName(u'OutlineCheckBox')
        self.outlineLayout = QtGui.QHBoxLayout()
        self.outlineLayout.setObjectName(u'OutlineLayout')
        self.outlineColorButton = QtGui.QPushButton(self.mainAreaPage)
        self.outlineColorButton.setEnabled(False)
        self.outlineColorButton.setObjectName(u'OutlineColorButton')
        self.outlineLayout.addWidget(self.outlineColorButton)
        self.outlineLayout.addSpacing(20)
        self.outlineSizeLabel = QtGui.QLabel(self.mainAreaPage)
        self.outlineSizeLabel.setObjectName(u'OutlineSizeLabel')
        self.outlineLayout.addWidget(self.outlineSizeLabel)
        self.outlineSizeSpinBox = QtGui.QSpinBox(self.mainAreaPage)
        self.outlineSizeSpinBox.setEnabled(False)
        self.outlineSizeSpinBox.setObjectName(u'OutlineSizeSpinBox')
        self.outlineLayout.addWidget(self.outlineSizeSpinBox)
        self.mainAreaLayout.addRow(self.outlineCheckBox, self.outlineLayout)
        self.shadowCheckBox = QtGui.QCheckBox(self.mainAreaPage)
        self.shadowCheckBox.setObjectName(u'ShadowCheckBox')
        self.shadowLayout = QtGui.QHBoxLayout()
        self.shadowLayout.setObjectName(u'ShadowLayout')
        self.shadowColorButton = QtGui.QPushButton(self.mainAreaPage)
        self.shadowColorButton.setEnabled(False)
        self.shadowColorButton.setObjectName(u'shadowColorButton')
        self.shadowLayout.addWidget(self.shadowColorButton)
        self.shadowLayout.addSpacing(20)
        self.shadowSizeLabel = QtGui.QLabel(self.mainAreaPage)
        self.shadowSizeLabel.setObjectName(u'ShadowSizeLabel')
        self.shadowLayout.addWidget(self.shadowSizeLabel)
        self.shadowSizeSpinBox = QtGui.QSpinBox(self.mainAreaPage)
        self.shadowSizeSpinBox.setEnabled(False)
        self.shadowSizeSpinBox.setObjectName(u'ShadowSizeSpinBox')
        self.shadowLayout.addWidget(self.shadowSizeSpinBox)
        self.mainAreaLayout.addRow(self.shadowCheckBox, self.shadowLayout)
        themeWizard.addPage(self.mainAreaPage)
        # Footer Area Page
        self.footerAreaPage = QtGui.QWizardPage()
        self.footerAreaPage.setObjectName(u'FooterAreaPage')
        self.footerAreaLayout = QtGui.QFormLayout(self.footerAreaPage)
        self.footerAreaLayout.setObjectName(u'FooterAreaLayout')
        self.footerFontLabel = QtGui.QLabel(self.footerAreaPage)
        self.footerFontLabel.setObjectName(u'FooterFontLabel')
        self.footerFontComboBox = QtGui.QFontComboBox(self.footerAreaPage)
        self.footerFontComboBox.setObjectName(u'footerFontComboBox')
        self.footerAreaLayout.addRow(self.footerFontLabel,
            self.footerFontComboBox)
        self.footerColorLabel = QtGui.QLabel(self.footerAreaPage)
        self.footerColorLabel.setObjectName(u'FooterColorLabel')
        self.footerColorButton = QtGui.QPushButton(self.footerAreaPage)
        self.footerColorButton.setObjectName(u'footerColorButton')
        self.footerAreaLayout.addRow(self.footerColorLabel,
            self.footerColorButton)
        self.footerSizeLabel = QtGui.QLabel(self.footerAreaPage)
        self.footerSizeLabel.setObjectName(u'FooterSizeLabel')
        self.footerSizeSpinBox = QtGui.QSpinBox(self.footerAreaPage)
        self.footerSizeSpinBox.setMaximum(999)
        self.footerSizeSpinBox.setValue(10)
        self.footerSizeSpinBox.setObjectName(u'FooterSizeSpinBox')
        self.footerAreaLayout.addRow(self.footerSizeLabel,
            self.footerSizeSpinBox)
        self.footerAreaLayout.setItem(3, QtGui.QFormLayout.LabelRole,
            self.spacer)
        themeWizard.addPage(self.footerAreaPage)
        # Alignment Page
        self.alignmentPage = QtGui.QWizardPage()
        self.alignmentPage.setObjectName(u'AlignmentPage')
        self.alignmentLayout = QtGui.QFormLayout(self.alignmentPage)
        self.alignmentLayout.setObjectName(u'AlignmentLayout')
        self.horizontalLabel = QtGui.QLabel(self.alignmentPage)
        self.horizontalLabel.setObjectName(u'HorizontalLabel')
        self.horizontalComboBox = QtGui.QComboBox(self.alignmentPage)
        self.horizontalComboBox.addItems([u'', u'', u'', u''])
        self.horizontalComboBox.setObjectName(u'HorizontalComboBox')
        self.alignmentLayout.addRow(self.horizontalLabel,
            self.horizontalComboBox)
        self.verticalLabel, self.verticalComboBox = \
            create_valign_selection_widgets(self.alignmentPage)
        self.verticalLabel.setObjectName(u'verticalLabel')
        self.verticalComboBox.setObjectName(u'verticalComboBox')
        self.alignmentLayout.addRow(self.verticalLabel, self.verticalComboBox)
        self.transitionsLabel = QtGui.QLabel(self.alignmentPage)
        self.transitionsLabel.setObjectName(u'TransitionsLabel')
        self.transitionsCheckBox = QtGui.QCheckBox(self.alignmentPage)
        self.transitionsCheckBox.setObjectName(u'TransitionsCheckBox')
        self.alignmentLayout.addRow(self.transitionsLabel,
            self.transitionsCheckBox)
        self.alignmentLayout.setItem(3, QtGui.QFormLayout.LabelRole,
            self.spacer)
        themeWizard.addPage(self.alignmentPage)
        # Area Position Page
        self.areaPositionPage = QtGui.QWizardPage()
        self.areaPositionPage.setObjectName(u'AreaPositionPage')
        self.areaPositionLayout = QtGui.QHBoxLayout(self.areaPositionPage)
        self.areaPositionLayout.setObjectName(u'AreaPositionLayout')
        self.mainPositionGroupBox = QtGui.QGroupBox(self.areaPositionPage)
        self.mainPositionGroupBox.setObjectName(u'MainPositionGroupBox')
        self.mainPositionLayout = QtGui.QFormLayout(self.mainPositionGroupBox)
        self.mainPositionLayout.setObjectName(u'MainPositionLayout')
        self.mainPositionCheckBox = QtGui.QCheckBox(self.mainPositionGroupBox)
        self.mainPositionCheckBox.setObjectName(u'MainPositionCheckBox')
        self.mainPositionLayout.addRow(self.mainPositionCheckBox)
        self.mainXLabel = QtGui.QLabel(self.mainPositionGroupBox)
        self.mainXLabel.setObjectName(u'MainXLabel')
        self.mainXSpinBox = QtGui.QSpinBox(self.mainPositionGroupBox)
        self.mainXSpinBox.setMaximum(9999)
        self.mainXSpinBox.setObjectName(u'MainXSpinBox')
        self.mainPositionLayout.addRow(self.mainXLabel, self.mainXSpinBox)
        self.mainYLabel = QtGui.QLabel(self.mainPositionGroupBox)
        self.mainYLabel.setObjectName(u'MainYLabel')
        self.mainYSpinBox = QtGui.QSpinBox(self.mainPositionGroupBox)
        self.mainYSpinBox.setMaximum(9999)
        self.mainYSpinBox.setObjectName(u'MainYSpinBox')
        self.mainPositionLayout.addRow(self.mainYLabel, self.mainYSpinBox)
        self.mainWidthLabel = QtGui.QLabel(self.mainPositionGroupBox)
        self.mainWidthLabel.setObjectName(u'MainWidthLabel')
        self.mainWidthSpinBox = QtGui.QSpinBox(self.mainPositionGroupBox)
        self.mainWidthSpinBox.setMaximum(9999)
        self.mainWidthSpinBox.setObjectName(u'MainWidthSpinBox')
        self.mainPositionLayout.addRow(self.mainWidthLabel,
            self.mainWidthSpinBox)
        self.mainHeightLabel = QtGui.QLabel(self.mainPositionGroupBox)
        self.mainHeightLabel.setObjectName(u'MainHeightLabel')
        self.mainHeightSpinBox = QtGui.QSpinBox(self.mainPositionGroupBox)
        self.mainHeightSpinBox.setMaximum(9999)
        self.mainHeightSpinBox.setObjectName(u'MainHeightSpinBox')
        self.mainPositionLayout.addRow(self.mainHeightLabel,
            self.mainHeightSpinBox)
        self.areaPositionLayout.addWidget(self.mainPositionGroupBox)
        self.footerPositionGroupBox = QtGui.QGroupBox(self.areaPositionPage)
        self.footerPositionGroupBox.setObjectName(u'FooterPositionGroupBox')
        self.footerPositionLayout = QtGui.QFormLayout(
            self.footerPositionGroupBox)
        self.footerPositionLayout.setObjectName(u'FooterPositionLayout')
        self.footerPositionCheckBox = QtGui.QCheckBox(
            self.footerPositionGroupBox)
        self.footerPositionCheckBox.setObjectName(u'FooterPositionCheckBox')
        self.footerPositionLayout.addRow(self.footerPositionCheckBox)
        self.footerXLabel = QtGui.QLabel(self.footerPositionGroupBox)
        self.footerXLabel.setObjectName(u'FooterXLabel')
        self.footerXSpinBox = QtGui.QSpinBox(self.footerPositionGroupBox)
        self.footerXSpinBox.setMaximum(9999)
        self.footerXSpinBox.setObjectName(u'FooterXSpinBox')
        self.footerPositionLayout.addRow(self.footerXLabel, self.footerXSpinBox)
        self.footerYLabel = QtGui.QLabel(self.footerPositionGroupBox)
        self.footerYLabel.setObjectName(u'FooterYLabel')
        self.footerYSpinBox = QtGui.QSpinBox(self.footerPositionGroupBox)
        self.footerYSpinBox.setMaximum(9999)
        self.footerYSpinBox.setObjectName(u'FooterYSpinBox')
        self.footerPositionLayout.addRow(self.footerYLabel, self.footerYSpinBox)
        self.footerWidthLabel = QtGui.QLabel(self.footerPositionGroupBox)
        self.footerWidthLabel.setObjectName(u'FooterWidthLabel')
        self.footerWidthSpinBox = QtGui.QSpinBox(self.footerPositionGroupBox)
        self.footerWidthSpinBox.setMaximum(9999)
        self.footerWidthSpinBox.setObjectName(u'FooterWidthSpinBox')
        self.footerPositionLayout.addRow(self.footerWidthLabel,
            self.footerWidthSpinBox)
        self.footerHeightLabel = QtGui.QLabel(self.footerPositionGroupBox)
        self.footerHeightLabel.setObjectName(u'FooterHeightLabel')
        self.footerHeightSpinBox = QtGui.QSpinBox(self.footerPositionGroupBox)
        self.footerHeightSpinBox.setMaximum(9999)
        self.footerHeightSpinBox.setObjectName(u'FooterHeightSpinBox')
        self.footerPositionLayout.addRow(self.footerHeightLabel,
            self.footerHeightSpinBox)
        self.areaPositionLayout.addWidget(self.footerPositionGroupBox)
        themeWizard.addPage(self.areaPositionPage)
        # Preview Page
        self.previewPage = QtGui.QWizardPage()
        self.previewPage.setObjectName(u'PreviewPage')
        self.previewLayout = QtGui.QVBoxLayout(self.previewPage)
        self.previewLayout.setObjectName(u'PreviewLayout')
        self.themeNameLayout = QtGui.QFormLayout()
        self.themeNameLayout.setObjectName(u'ThemeNameLayout')
        self.themeNameLabel = QtGui.QLabel(self.previewPage)
        self.themeNameLabel.setObjectName(u'ThemeNameLabel')
        self.themeNameEdit = QtGui.QLineEdit(self.previewPage)
        self.themeNameEdit.setValidator(QtGui.QRegExpValidator(
            QtCore.QRegExp(r'[^/\\?*|<>\[\]":<>+%]+'), self))
        self.themeNameEdit.setObjectName(u'ThemeNameEdit')
        self.themeNameLayout.addRow(self.themeNameLabel, self.themeNameEdit)
        self.previewLayout.addLayout(self.themeNameLayout)
        self.previewArea = QtGui.QWidget(self.previewPage)
        self.previewArea.setObjectName(u'PreviewArea')
        self.previewAreaLayout = QtGui.QGridLayout(self.previewArea)
        self.previewAreaLayout.setMargin(0)
        self.previewAreaLayout.setColumnStretch(0, 1)
        self.previewAreaLayout.setRowStretch(0, 1)
        self.previewAreaLayout.setObjectName(u'PreviewAreaLayout')
        self.previewBoxLabel = QtGui.QLabel(self.previewArea)
        self.previewBoxLabel.setFrameShape(QtGui.QFrame.Box)
        self.previewBoxLabel.setScaledContents(True)
        self.previewBoxLabel.setObjectName(u'PreviewBoxLabel')
        self.previewAreaLayout.addWidget(self.previewBoxLabel)
        self.previewLayout.addWidget(self.previewArea)
        themeWizard.addPage(self.previewPage)
        self.retranslateUi(themeWizard)
        QtCore.QObject.connect(self.backgroundComboBox,
            QtCore.SIGNAL(u'currentIndexChanged(int)'), self.backgroundStack,
            QtCore.SLOT(u'setCurrentIndex(int)'))
        QtCore.QObject.connect(self.outlineCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.outlineColorButton,
            QtCore.SLOT(u'setEnabled(bool)'))
        QtCore.QObject.connect(self.outlineCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.outlineSizeSpinBox,
            QtCore.SLOT(u'setEnabled(bool)'))
        QtCore.QObject.connect(self.shadowCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.shadowColorButton,
            QtCore.SLOT(u'setEnabled(bool)'))
        QtCore.QObject.connect(self.shadowCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.shadowSizeSpinBox,
            QtCore.SLOT(u'setEnabled(bool)'))
        QtCore.QObject.connect(self.mainPositionCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.mainXSpinBox,
            QtCore.SLOT(u'setDisabled(bool)'))
        QtCore.QObject.connect(self.mainPositionCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.mainYSpinBox,
            QtCore.SLOT(u'setDisabled(bool)'))
        QtCore.QObject.connect(self.mainPositionCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.mainWidthSpinBox,
            QtCore.SLOT(u'setDisabled(bool)'))
        QtCore.QObject.connect(self.mainPositionCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.mainHeightSpinBox,
            QtCore.SLOT(u'setDisabled(bool)'))
        QtCore.QObject.connect(self.footerPositionCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.footerXSpinBox,
            QtCore.SLOT(u'setDisabled(bool)'))
        QtCore.QObject.connect(self.footerPositionCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.footerYSpinBox,
            QtCore.SLOT(u'setDisabled(bool)'))
        QtCore.QObject.connect(self.footerPositionCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.footerWidthSpinBox,
            QtCore.SLOT(u'setDisabled(bool)'))
        QtCore.QObject.connect(self.footerPositionCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.footerHeightSpinBox,
            QtCore.SLOT(u'setDisabled(bool)'))

    def retranslateUi(self, themeWizard):
        themeWizard.setWindowTitle(
            translate('OpenLP.ThemeWizard', 'Theme Wizard'))
        self.titleLabel.setText(
            u'<span style="font-size:14pt; font-weight:600;">%s</span>' % \
            translate('OpenLP.ThemeWizard', 'Welcome to the Theme Wizard'))
        self.informationLabel.setText(
            translate('OpenLP.ThemeWizard', 'This wizard will help you to '
                'create and edit your themes. Click the next button below to '
                'start the process by setting up your background.'))
        self.backgroundPage.setTitle(
            translate('OpenLP.ThemeWizard', 'Set Up Background'))
        self.backgroundPage.setSubTitle(
            translate('OpenLP.ThemeWizard', 'Set up your theme\'s background '
                'according to the parameters below.'))
        self.backgroundLabel.setText(
            translate('OpenLP.ThemeWizard', 'Background type:'))
        self.backgroundComboBox.setItemText(BackgroundType.Solid,
            translate('OpenLP.ThemeWizard', 'Solid Color'))
        self.backgroundComboBox.setItemText(BackgroundType.Gradient,
            translate('OpenLP.ThemeWizard', 'Gradient'))
        self.backgroundComboBox.setItemText(
            BackgroundType.Image, UiStrings().Image)
        self.backgroundComboBox.setItemText(BackgroundType.Transparent,
            translate('OpenLP.ThemeWizard', 'Transparent'))
        self.colorLabel.setText(translate('OpenLP.ThemeWizard', 'Color:'))
        self.gradientStartLabel.setText(
            translate(u'OpenLP.ThemeWizard', 'Starting color:'))
        self.gradientEndLabel.setText(
            translate(u'OpenLP.ThemeWizard', 'Ending color:'))
        self.gradientTypeLabel.setText(
            translate('OpenLP.ThemeWizard', 'Gradient:'))
        self.gradientComboBox.setItemText(BackgroundGradientType.Horizontal,
            translate('OpenLP.ThemeWizard', 'Horizontal'))
        self.gradientComboBox.setItemText(BackgroundGradientType.Vertical,
            translate('OpenLP.ThemeWizard', 'Vertical'))
        self.gradientComboBox.setItemText(BackgroundGradientType.Circular,
            translate('OpenLP.ThemeWizard', 'Circular'))
        self.gradientComboBox.setItemText(BackgroundGradientType.LeftTop,
            translate('OpenLP.ThemeWizard', 'Top Left - Bottom Right'))
        self.gradientComboBox.setItemText(BackgroundGradientType.LeftBottom,
            translate('OpenLP.ThemeWizard', 'Bottom Left - Top Right'))
        self.imageColorLabel.setText(
            translate(u'OpenLP.ThemeWizard', 'Background color:'))
        self.imageLabel.setText(u'%s:' % UiStrings().Image)
        self.mainAreaPage.setTitle(
            translate('OpenLP.ThemeWizard', 'Main Area Font Details'))
        self.mainAreaPage.setSubTitle(
            translate('OpenLP.ThemeWizard', 'Define the font and display '
                'characteristics for the Display text'))
        self.mainFontLabel.setText(translate('OpenLP.ThemeWizard', 'Font:'))
        self.mainColorLabel.setText(translate('OpenLP.ThemeWizard', 'Color:'))
        self.mainSizeLabel.setText(translate('OpenLP.ThemeWizard', 'Size:'))
        self.mainSizeSpinBox.setSuffix(UiStrings().FontSizePtUnit)
        self.lineSpacingLabel.setText(
            translate('OpenLP.ThemeWizard', 'Line Spacing:'))
        self.lineSpacingSpinBox.setSuffix(UiStrings().FontSizePtUnit)
        self.outlineCheckBox.setText(
            translate('OpenLP.ThemeWizard', '&Outline:'))
        self.outlineSizeLabel.setText(translate('OpenLP.ThemeWizard', 'Size:'))
        self.outlineSizeSpinBox.setSuffix(UiStrings().FontSizePtUnit)
        self.shadowCheckBox.setText(translate('OpenLP.ThemeWizard', '&Shadow:'))
        self.shadowSizeLabel.setText(translate('OpenLP.ThemeWizard', 'Size:'))
        self.shadowSizeSpinBox.setSuffix(UiStrings().FontSizePtUnit)
        self.mainBoldCheckBox.setText(translate('OpenLP.ThemeWizard', 'Bold'))
        self.mainItalicsCheckBox.setText(
            translate('OpenLP.ThemeWizard', 'Italic'))
        self.footerAreaPage.setTitle(
            translate('OpenLP.ThemeWizard', 'Footer Area Font Details'))
        self.footerAreaPage.setSubTitle(
            translate('OpenLP.ThemeWizard', 'Define the font and display '
                'characteristics for the Footer text'))
        self.footerFontLabel.setText(translate('OpenLP.ThemeWizard', 'Font:'))
        self.footerColorLabel.setText(translate('OpenLP.ThemeWizard', 'Color:'))
        self.footerSizeLabel.setText(translate('OpenLP.ThemeWizard', 'Size:'))
        self.footerSizeSpinBox.setSuffix(UiStrings().FontSizePtUnit)
        self.alignmentPage.setTitle(
            translate('OpenLP.ThemeWizard', 'Text Formatting Details'))
        self.alignmentPage.setSubTitle(
            translate('OpenLP.ThemeWizard', 'Allows additional display '
                'formatting information to be defined'))
        self.horizontalLabel.setText(
            translate('OpenLP.ThemeWizard', 'Horizontal Align:'))
        self.horizontalComboBox.setItemText(HorizontalType.Left,
            translate('OpenLP.ThemeWizard', 'Left'))
        self.horizontalComboBox.setItemText(HorizontalType.Right,
            translate('OpenLP.ThemeWizard', 'Right'))
        self.horizontalComboBox.setItemText(HorizontalType.Center,
            translate('OpenLP.ThemeWizard', 'Center'))
        self.horizontalComboBox.setItemText(HorizontalType.Justify,
            translate('OpenLP.ThemeWizard', 'Justify'))
        self.transitionsLabel.setText(
            translate('OpenLP.ThemeWizard', 'Transitions:'))
        self.areaPositionPage.setTitle(
            translate('OpenLP.ThemeWizard', 'Output Area Locations'))
        self.areaPositionPage.setSubTitle(
            translate('OpenLP.ThemeWizard', 'Allows you to change and move the'
                ' main and footer areas.'))
        self.mainPositionGroupBox.setTitle(
            translate('OpenLP.ThemeWizard', '&Main Area'))
        self.mainPositionCheckBox.setText(
            translate('OpenLP.ThemeWizard', '&Use default location'))
        self.mainXLabel.setText(translate('OpenLP.ThemeWizard', 'X position:'))
        self.mainXSpinBox.setSuffix(translate('OpenLP.ThemeWizard', 'px'))
        self.mainYSpinBox.setSuffix(translate('OpenLP.ThemeWizard', 'px'))
        self.mainYLabel.setText(translate('OpenLP.ThemeWizard', 'Y position:'))
        self.mainWidthSpinBox.setSuffix(translate('OpenLP.ThemeWizard', 'px'))
        self.mainWidthLabel.setText(translate('OpenLP.ThemeWizard', 'Width:'))
        self.mainHeightSpinBox.setSuffix(translate('OpenLP.ThemeWizard', 'px'))
        self.mainHeightLabel.setText(
            translate('OpenLP.ThemeWizard', 'Height:'))
        self.footerPositionGroupBox.setTitle(
            translate('OpenLP.ThemeWizard', '&Footer Area'))
        self.footerXLabel.setText(
            translate('OpenLP.ThemeWizard', 'X position:'))
        self.footerXSpinBox.setSuffix(translate('OpenLP.ThemeWizard', 'px'))
        self.footerYLabel.setText(
            translate('OpenLP.ThemeWizard', 'Y position:'))
        self.footerYSpinBox.setSuffix(translate('OpenLP.ThemeWizard', 'px'))
        self.footerWidthLabel.setText(
            translate('OpenLP.ThemeWizard', 'Width:'))
        self.footerWidthSpinBox.setSuffix(
            translate('OpenLP.ThemeWizard', 'px'))
        self.footerHeightLabel.setText(
            translate('OpenLP.ThemeWizard', 'Height:'))
        self.footerHeightSpinBox.setSuffix(
            translate('OpenLP.ThemeWizard', 'px'))
        self.footerPositionCheckBox.setText(
            translate('OpenLP.ThemeWizard', 'Use default location'))
        themeWizard.setOption(QtGui.QWizard.HaveCustomButton1, False)
        themeWizard.setButtonText(QtGui.QWizard.CustomButton1,
            translate('OpenLP.ThemeWizard', 'Layout Preview'))
        self.previewPage.setTitle(
            translate('OpenLP.ThemeWizard', 'Preview and Save'))
        self.previewPage.setSubTitle(
            translate('OpenLP.ThemeWizard', 'Preview the theme and save it.'))
        self.themeNameLabel.setText(
            translate('OpenLP.ThemeWizard', 'Theme name:'))
        # Align all QFormLayouts towards each other.
        labelWidth = max(self.backgroundLabel.minimumSizeHint().width(),
            self.horizontalLabel.minimumSizeHint().width())
        self.spacer.changeSize(labelWidth, 0,
            QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging
import mimetypes
from datetime import datetime

from PyQt4.phonon import Phonon

from openlp.core.lib import Receiver
from openlp.core.lib.mediaplayer import MediaPlayer
from openlp.core.ui.media import MediaState

log = logging.getLogger(__name__)

ADDITIONAL_EXT = {
        u'audio/ac3': [u'.ac3'],
        u'audio/flac': [u'.flac'],
        u'audio/x-m4a': [u'.m4a'],
        u'audio/midi': [u'.mid', u'.midi'],
        u'audio/x-mp3': [u'.mp3'],
        u'audio/mpeg': [u'.mp3', u'.mp2', u'.mpga', u'.mpega', u'.m4a'],
        u'audio/qcelp': [u'.qcp'],
        u'audio/x-wma': [u'.wma'],
        u'audio/x-ms-wma': [u'.wma'],
        u'video/x-flv': [u'.flv'],
        u'video/x-matroska': [u'.mpv', u'.mkv'],
        u'video/x-wmv': [u'.wmv'],
        u'video/x-mpg': [u'.mpg'],
        u'video/mpeg' : [u'.mp4', u'.mts', u'.mov'],
        u'video/x-ms-wmv': [u'.wmv']}


class PhononPlayer(MediaPlayer):
    """
    A specialised version of the MediaPlayer class, which provides a Phonon
    display.
    """

    def __init__(self, parent):
        MediaPlayer.__init__(self, parent, u'phonon')
        self.original_name = u'Phonon'
        self.display_name = u'&Phonon'
        self.parent = parent
        self.additional_extensions = ADDITIONAL_EXT
        try:
            mimetypes.init()
            for mimetype in Phonon.BackendCapabilities.availableMimeTypes():
                mimetype = unicode(mimetype)
                if mimetype.startswith(u'audio/'):
                    self._addToList(self.audio_extensions_list, mimetype)
                elif mimetype.startswith(u'video/'):
                    self._addToList(self.video_extensions_list, mimetype)
        # Work around for http://bugs.python.org/issue9291
        except UnicodeDecodeError:
            log.exception(u'UnicodeDecodeError when trying to read mime types. '
                'Setting extensions to all files')
            self.audio_extensions_list = [u'*.*']
            self.video_extensions_list = [u'*.*']

    def _addToList(self, list, mimetype):
        # Add all extensions which mimetypes provides us for supported types.
        extensions = mimetypes.guess_all_extensions(unicode(mimetype))
        for extension in extensions:
            ext = u'*%s' % extension
            if ext not in list:
                list.append(ext)
        log.info(u'MediaPlugin: %s extensions: %s' % (mimetype,
            u' '.join(extensions)))
        # Add extensions for this mimetype from self.additional_extensions.
        # This hack clears mimetypes' and operating system's shortcomings
        # by providing possibly missing extensions.
        if mimetype in self.additional_extensions.keys():
            for extension in self.additional_extensions[mimetype]:
                ext = u'*%s' % extension
                if ext not in list:
                    list.append(ext)
            log.info(u'MediaPlugin: %s additional extensions: %s' % (mimetype,
                u' '.join(self.additional_extensions[mimetype])))

    def setup(self, display):
        display.phononWidget = Phonon.VideoWidget(display)
        display.phononWidget.resize(display.size())
        display.mediaObject = Phonon.MediaObject(display)
        Phonon.createPath(display.mediaObject, display.phononWidget)
        if display.hasAudio:
            display.audio = Phonon.AudioOutput(
                Phonon.VideoCategory, display.mediaObject)
            Phonon.createPath(display.mediaObject, display.audio)
        display.phononWidget.raise_()
        display.phononWidget.hide()
        self.hasOwnWidget = True

    def check_available(self):
        return True

    def load(self, display):
        log.debug(u'load vid in Phonon Controller')
        controller = display.controller
        volume = controller.media_info.volume
        path = controller.media_info.file_info.absoluteFilePath()
        display.mediaObject.setCurrentSource(Phonon.MediaSource(path))
        if not self.media_state_wait(display, Phonon.StoppedState):
            return False
        self.volume(display, volume)
        return True

    def media_state_wait(self, display, mediaState):
        """
        Wait for the video to change its state
        Wait no longer than 5 seconds.
        """
        start = datetime.now()
        current_state = display.mediaObject.state()
        while current_state != mediaState:
            current_state = display.mediaObject.state()
            if current_state == Phonon.ErrorState:
                return False
            Receiver.send_message(u'openlp_process_events')
            if (datetime.now() - start).seconds > 5:
                return False
        return True

    def resize(self, display):
        display.phononWidget.resize(display.size())

    def play(self, display):
        controller = display.controller
        start_time = 0
        if display.mediaObject.state() != Phonon.PausedState and \
            controller.media_info.start_time > 0:
            start_time = controller.media_info.start_time
        display.mediaObject.play()
        if not self.media_state_wait(display, Phonon.PlayingState):
            return False
        if start_time > 0:
            self.seek(display, controller.media_info.start_time * 1000)
        self.volume(display, controller.media_info.volume)
        controller.media_info.length = \
            int(display.mediaObject.totalTime() / 1000)
        controller.seekSlider.setMaximum(controller.media_info.length * 1000)
        self.state = MediaState.Playing
        display.phononWidget.raise_()
        return True

    def pause(self, display):
        display.mediaObject.pause()
        if self.media_state_wait(display, Phonon.PausedState):
            self.state = MediaState.Paused

    def stop(self, display):
        display.mediaObject.stop()
        self.set_visible(display, False)
        self.state = MediaState.Stopped

    def volume(self, display, vol):
        # 1.0 is the highest value
        if display.hasAudio:
            vol = float(vol) / float(100)
            display.audio.setVolume(vol)

    def seek(self, display, seekVal):
        display.mediaObject.seek(seekVal)

    def reset(self, display):
        display.mediaObject.stop()
        display.mediaObject.clearQueue()
        self.set_visible(display, False)
        display.phononWidget.setVisible(False)
        self.state = MediaState.Off

    def set_visible(self, display, status):
        if self.hasOwnWidget:
            display.phononWidget.setVisible(status)

    def update_ui(self, display):
        if display.mediaObject.state() == Phonon.PausedState and \
            self.state != MediaState.Paused:
            self.stop(display)
        controller = display.controller
        if controller.media_info.end_time > 0:
            if display.mediaObject.currentTime() > \
                controller.media_info.end_time * 1000:
                self.stop(display)
                self.set_visible(display, False)
        if not controller.seekSlider.isSliderDown():
            controller.seekSlider.setSliderPosition(
                display.mediaObject.currentTime())



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging
import re
import os
import shutil

from PyQt4 import QtCore, QtGui
from sqlalchemy.sql import or_

from openlp.core.lib import MediaManagerItem, Receiver, ItemCapabilities, \
    translate, check_item_selected, PluginStatus, create_separated_list, \
    check_directory_exists
from openlp.core.lib.ui import UiStrings, create_widget_action
from openlp.core.lib.settings import Settings
from openlp.core.utils import AppLocation
from openlp.plugins.songs.forms import EditSongForm, SongMaintenanceForm, \
    SongImportForm, SongExportForm
from openlp.plugins.songs.lib import OpenLyrics, SongXML, VerseType, \
    clean_string, natcmp
from openlp.plugins.songs.lib.db import Author, Song, Book, MediaFile
from openlp.plugins.songs.lib.ui import SongStrings

log = logging.getLogger(__name__)


class SongSearch(object):
    """
    An enumeration for song search methods.
    """
    Entire = 1
    Titles = 2
    Lyrics = 3
    Authors = 4
    Books = 5
    Themes = 6


class SongMediaItem(MediaManagerItem):
    """
    This is the custom media manager item for Songs.
    """
    log.info(u'Song Media Item loaded')

    def __init__(self, parent, plugin, icon):
        self.IconPath = u'songs/song'
        MediaManagerItem.__init__(self, parent, plugin, icon)
        self.editSongForm = EditSongForm(self, self.plugin.formParent,
            self.plugin.manager)
        self.openLyrics = OpenLyrics(self.plugin.manager)
        self.singleServiceItem = False
        self.songMaintenanceForm = SongMaintenanceForm(
            self.plugin.manager, self)
        # Holds information about whether the edit is remotly triggered and
        # which Song is required.
        self.remoteSong = -1
        self.editItem = None
        self.quickPreviewAllowed = True
        self.hasSearch = True

    def _updateBackgroundAudio(self, song, item):
        song.media_files = []
        for i, bga in enumerate(item.background_audio):
            dest_file = os.path.join(
                AppLocation.get_section_data_path(self.plugin.name),
                u'audio', str(song.id), os.path.split(bga)[1])
            check_directory_exists(os.path.split(dest_file)[0])
            shutil.copyfile(os.path.join(
                AppLocation.get_section_data_path(
                    u'servicemanager'), bga),
                dest_file)
            song.media_files.append(MediaFile.populate(
                weight=i, file_name=dest_file))
        self.plugin.manager.save_object(song, True)

    def addEndHeaderBar(self):
        self.toolbar.addSeparator()
        ## Song Maintenance Button ##
        self.maintenanceAction = self.toolbar.addToolbarAction(
            u'maintenanceAction', icon=':/songs/song_maintenance.png',
            triggers=self.onSongMaintenanceClick)
        self.addSearchToToolBar()
        # Signals and slots
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'songs_load_list'), self.onSongListLoad)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'config_updated'), self.configUpdated)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'songs_preview'), self.onPreviewClick)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'songs_edit'), self.onRemoteEdit)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'songs_edit_clear'), self.onRemoteEditClear)
        QtCore.QObject.connect(self.searchTextEdit,
            QtCore.SIGNAL(u'cleared()'), self.onClearTextButtonClick)
        QtCore.QObject.connect(self.searchTextEdit,
            QtCore.SIGNAL(u'searchTypeChanged(int)'),
            self.onSearchTextButtonClicked)

    def addCustomContextActions(self):
        create_widget_action(self.listView, separator=True)
        create_widget_action(self.listView,
            text=translate('OpenLP.MediaManagerItem', '&Clone'),
            icon=u':/general/general_clone.png', triggers=self.onCloneClick)

    def onFocus(self):
        self.searchTextEdit.setFocus()

    def configUpdated(self):
        self.searchAsYouType = Settings().value(
            self.settingsSection + u'/search as type',
            QtCore.QVariant(u'False')).toBool()
        self.updateServiceOnEdit = Settings().value(
            self.settingsSection + u'/update service on edit',
            QtCore.QVariant(u'False')).toBool()
        self.addSongFromService = Settings().value(
            self.settingsSection + u'/add song from service',
            QtCore.QVariant(u'True')).toBool()

    def retranslateUi(self):
        self.searchTextLabel.setText(u'%s:' % UiStrings().Search)
        self.searchTextButton.setText(UiStrings().Search)
        self.maintenanceAction.setText(SongStrings.SongMaintenance)
        self.maintenanceAction.setToolTip(translate('SongsPlugin.MediaItem',
            'Maintain the lists of authors, topics and books.'))

    def initialise(self):
        self.searchTextEdit.setSearchTypes([
            (SongSearch.Entire, u':/songs/song_search_all.png',
                translate('SongsPlugin.MediaItem', 'Entire Song'),
                translate('SongsPlugin.MediaItem', 'Search Entire Song...')),
            (SongSearch.Titles, u':/songs/song_search_title.png',
                translate('SongsPlugin.MediaItem', 'Titles'),
                translate('SongsPlugin.MediaItem', 'Search Titles...')),
            (SongSearch.Lyrics, u':/songs/song_search_lyrics.png',
                translate('SongsPlugin.MediaItem', 'Lyrics'),
                translate('SongsPlugin.MediaItem', 'Search Lyrics...')),
            (SongSearch.Authors, u':/songs/song_search_author.png',
                SongStrings.Authors,
                translate('SongsPlugin.MediaItem', 'Search Authors...')),
            (SongSearch.Books, u':/songs/song_book_edit.png',
                SongStrings.SongBooks,
                translate('SongsPlugin.MediaItem', 'Search Song Books...')),
            (SongSearch.Themes, u':/slides/slide_theme.png',
            UiStrings().Themes, UiStrings().SearchThemes)
        ])
        self.searchTextEdit.setCurrentSearchType(Settings().value(
            u'%s/last search type' % self.settingsSection,
            QtCore.QVariant(SongSearch.Entire)).toInt()[0])
        self.configUpdated()

    def onSearchTextButtonClicked(self):
        # Save the current search type to the configuration.
        Settings().setValue(u'%s/last search type' %
            self.settingsSection,
            QtCore.QVariant(self.searchTextEdit.currentSearchType()))
        # Reload the list considering the new search type.
        search_keywords = unicode(self.searchTextEdit.displayText())
        search_results = []
        search_type = self.searchTextEdit.currentSearchType()
        if search_type == SongSearch.Entire:
            log.debug(u'Entire Song Search')
            search_results = self.searchEntire(search_keywords)
            self.displayResultsSong(search_results)
        elif search_type == SongSearch.Titles:
            log.debug(u'Titles Search')
            search_results = self.plugin.manager.get_all_objects(Song,
                Song.search_title.like(u'%' + clean_string(search_keywords) +
                    u'%'))
            self.displayResultsSong(search_results)
        elif search_type == SongSearch.Lyrics:
            log.debug(u'Lyrics Search')
            search_results = self.plugin.manager.get_all_objects(Song,
                Song.search_lyrics.like(u'%' + clean_string(search_keywords) +
                    u'%'))
            self.displayResultsSong(search_results)
        elif search_type == SongSearch.Authors:
            log.debug(u'Authors Search')
            search_results = self.plugin.manager.get_all_objects(Author,
                Author.display_name.like(u'%' + search_keywords + u'%'),
                Author.display_name.asc())
            self.displayResultsAuthor(search_results)
        elif search_type == SongSearch.Books:
            log.debug(u'Books Search')
            search_results = self.plugin.manager.get_all_objects(Book,
                Book.name.like(u'%' + search_keywords + u'%'),
                Book.name.asc())
            song_number = False
            if not search_results:
                search_keywords = search_keywords.rpartition(' ')
                search_results = self.plugin.manager.get_all_objects(Book,
                    Book.name.like(u'%' + search_keywords[0] + u'%'),
                    Book.name.asc())
                song_number = re.sub(r'[^0-9]', u'', search_keywords[2])
            self.displayResultsBook(search_results, song_number)
        elif search_type == SongSearch.Themes:
            log.debug(u'Theme Search')
            search_results = self.plugin.manager.get_all_objects(Song,
                Song.theme_name.like(u'%' + search_keywords + u'%'))
            self.displayResultsSong(search_results)
        self.checkSearchResult()

    def searchEntire(self, search_keywords):
        return self.plugin.manager.get_all_objects(Song,
            or_(Song.search_title.like(u'%' + clean_string(search_keywords)
                + u'%'),
            Song.search_lyrics.like(u'%' + clean_string(search_keywords)
                + u'%'),
            Song.comments.like(u'%' + search_keywords.lower() + u'%')))

    def onSongListLoad(self):
        """
        Handle the exit from the edit dialog and trigger remote updates
        of songs
        """
        log.debug(u'onSongListLoad - start')
        # Called to redisplay the song list screen edit from a search
        # or from the exit of the Song edit dialog. If remote editing is active
        # Trigger it and clean up so it will not update again.
        if self.remoteTriggered == u'L':
            self.onAddClick()
        if self.remoteTriggered == u'P':
            self.onPreviewClick()
        # Push edits to the service manager to update items
        if self.editItem and self.updateServiceOnEdit and \
            not self.remoteTriggered:
            item = self.buildServiceItem(self.editItem)
            self.plugin.serviceManager.replaceServiceItem(item)
        self.onRemoteEditClear()
        self.onSearchTextButtonClicked()
        log.debug(u'onSongListLoad - finished')

    def displayResultsSong(self, searchresults):
        log.debug(u'display results Song')
        self.saveAutoSelectId()
        self.listView.clear()
        searchresults.sort(cmp=natcmp, key=lambda song: song.sort_key)
        for song in searchresults:
            # Do not display temporary songs
            if song.temporary:
                continue
            author_list = [author.display_name for author in song.authors]
            song_title = unicode(song.title)
            song_detail = u'%s (%s)' % (song_title,
                create_separated_list(author_list))
            song_name = QtGui.QListWidgetItem(song_detail)
            song_name.setData(QtCore.Qt.UserRole, QtCore.QVariant(song.id))
            self.listView.addItem(song_name)
            # Auto-select the item if name has been set
            if song.id == self.autoSelectId:
                self.listView.setCurrentItem(song_name)
        self.autoSelectId = -1

    def displayResultsAuthor(self, searchresults):
        log.debug(u'display results Author')
        self.listView.clear()
        for author in searchresults:
            for song in author.songs:
                # Do not display temporary songs
                if song.temporary:
                    continue
                song_detail = u'%s (%s)' % (author.display_name, song.title)
                song_name = QtGui.QListWidgetItem(song_detail)
                song_name.setData(QtCore.Qt.UserRole, QtCore.QVariant(song.id))
                self.listView.addItem(song_name)

    def displayResultsBook(self, searchresults, song_number=False):
        log.debug(u'display results Book')
        self.listView.clear()
        for book in searchresults:
            songs = sorted(book.songs, key=lambda song:
                int(re.match(r'[0-9]+', u'0' + song.song_number).group()))
            for song in songs:
                # Do not display temporary songs
                if song.temporary:
                    continue
                if song_number and not song_number in song.song_number:
                    continue
                song_detail = u'%s - %s (%s)' % (book.name, song.song_number,
                    song.title)
                song_name = QtGui.QListWidgetItem(song_detail)
                song_name.setData(QtCore.Qt.UserRole, QtCore.QVariant(song.id))
                self.listView.addItem(song_name)

    def onClearTextButtonClick(self):
        """
        Clear the search text.
        """
        self.searchTextEdit.clear()
        self.onSearchTextButtonClicked()

    def onSearchTextEditChanged(self, text):
        """
        If search as type enabled invoke the search on each key press.
        If the Lyrics are being searched do not start till 7 characters
        have been entered.
        """
        if self.searchAsYouType:
            search_length = 1
            if self.searchTextEdit.currentSearchType() == SongSearch.Entire:
                search_length = 4
            elif self.searchTextEdit.currentSearchType() == SongSearch.Lyrics:
                search_length = 3
            if len(text) > search_length:
                self.onSearchTextButtonClicked()
            elif not text:
                self.onClearTextButtonClick()

    def onImportClick(self):
        if not hasattr(self, u'importWizard'):
            self.importWizard = SongImportForm(self, self.plugin)
        self.importWizard.exec_()
        # Run song load as list may have been cancelled but some songs loaded
        Receiver.send_message(u'songs_load_list')

    def onExportClick(self):
        if not hasattr(self, u'exportWizard'):
            self.exportWizard = SongExportForm(self, self.plugin)
        self.exportWizard.exec_()

    def onNewClick(self):
        log.debug(u'onNewClick')
        self.editSongForm.newSong()
        self.editSongForm.exec_()
        self.onClearTextButtonClick()
        self.onSelectionChange()
        self.autoSelectId = -1

    def onSongMaintenanceClick(self):
        self.songMaintenanceForm.exec_()

    def onRemoteEditClear(self):
        log.debug(u'onRemoteEditClear')
        self.remoteTriggered = None
        self.remoteSong = -1

    def onRemoteEdit(self, message):
        """
        Called by ServiceManager or SlideController by event passing
        the Song Id in the payload along with an indicator to say which
        type of display is required.
        """
        log.debug(u'onRemoteEdit %s' % message)
        remote_type, song_id = message.split(u':')
        song_id = int(song_id)
        valid = self.plugin.manager.get_object(Song, song_id)
        if valid:
            self.remoteSong = song_id
            self.remoteTriggered = remote_type
            self.editSongForm.loadSong(song_id, remote_type == u'P')
            self.editSongForm.exec_()
            self.autoSelectId = -1
            self.onSongListLoad()

    def onEditClick(self):
        """
        Edit a song
        """
        log.debug(u'onEditClick')
        if check_item_selected(self.listView, UiStrings().SelectEdit):
            self.editItem = self.listView.currentItem()
            item_id = (self.editItem.data(QtCore.Qt.UserRole)).toInt()[0]
            self.editSongForm.loadSong(item_id, False)
            self.editSongForm.exec_()
            self.autoSelectId = -1
            self.onSongListLoad()
        self.editItem = None

    def onDeleteClick(self):
        """
        Remove a song from the list and database
        """
        if check_item_selected(self.listView, UiStrings().SelectDelete):
            items = self.listView.selectedIndexes()
            if QtGui.QMessageBox.question(self,
                UiStrings().ConfirmDelete,
                translate('SongsPlugin.MediaItem',
                'Are you sure you want to delete the %n selected song(s)?', '',
                QtCore.QCoreApplication.CodecForTr, len(items)),
                QtGui.QMessageBox.StandardButtons(QtGui.QMessageBox.Yes |
                QtGui.QMessageBox.No),
                QtGui.QMessageBox.Yes) == QtGui.QMessageBox.No:
                return
            Receiver.send_message(u'cursor_busy')
            self.plugin.formParent.displayProgressBar(len(items))
            for item in items:
                item_id = (item.data(QtCore.Qt.UserRole)).toInt()[0]
                media_files = self.plugin.manager.get_all_objects(MediaFile,
                    MediaFile.song_id == item_id)
                for media_file in media_files:
                    try:
                        os.remove(media_file.file_name)
                    except:
                        log.exception('Could not remove file: %s',
                            media_file.file_name)
                try:
                    save_path = os.path.join(AppLocation.get_section_data_path(
                        self.plugin.name), 'audio', str(item_id))
                    if os.path.exists(save_path):
                        os.rmdir(save_path)
                except OSError:
                    log.exception(u'Could not remove directory: %s', save_path)
                self.plugin.manager.delete_object(Song, item_id)
                self.plugin.formParent.incrementProgressBar()
            self.plugin.formParent.finishedProgressBar()
            Receiver.send_message(u'cursor_normal')
            self.onSearchTextButtonClicked()

    def onCloneClick(self):
        """
        Clone a Song
        """
        log.debug(u'onCloneClick')
        if check_item_selected(self.listView, UiStrings().SelectEdit):
            self.editItem = self.listView.currentItem()
            item_id = (self.editItem.data(QtCore.Qt.UserRole)).toInt()[0]
            old_song = self.plugin.manager.get_object(Song, item_id)
            song_xml = self.openLyrics.song_to_xml(old_song)
            new_song = self.openLyrics.xml_to_song(song_xml)
            new_song.title = u'%s <%s>' % (new_song.title,
                translate('SongsPlugin.MediaItem', 'copy',
                'For song cloning'))
            self.plugin.manager.save_object(new_song)
        self.onSongListLoad()

    def generateSlideData(self, service_item, item=None, xmlVersion=False,
        remote=False):
        log.debug(u'generateSlideData: %s, %s, %s' %
            (service_item, item, self.remoteSong))
        item_id = self._getIdOfItemToGenerate(item, self.remoteSong)
        service_item.add_capability(ItemCapabilities.CanEdit)
        service_item.add_capability(ItemCapabilities.CanPreview)
        service_item.add_capability(ItemCapabilities.CanLoop)
        service_item.add_capability(ItemCapabilities.OnLoadUpdate)
        service_item.add_capability(ItemCapabilities.AddIfNewItem)
        service_item.add_capability(ItemCapabilities.CanSoftBreak)
        song = self.plugin.manager.get_object(Song, item_id)
        service_item.theme = song.theme_name
        service_item.edit_id = item_id
        if song.lyrics.startswith(u'<?xml version='):
            verse_list = SongXML().get_verses(song.lyrics)
            # no verse list or only 1 space (in error)
            verse_tags_translated = False
            if VerseType.from_translated_string(unicode(
                verse_list[0][0][u'type'])) is not None:
                verse_tags_translated = True
            if not song.verse_order.strip():
                for verse in verse_list:
                    # We cannot use from_loose_input() here, because database
                    # is supposed to contain English lowercase singlechar tags.
                    verse_tag = verse[0][u'type']
                    verse_index = None
                    if len(verse_tag) > 1:
                        verse_index = \
                            VerseType.from_translated_string(verse_tag)
                        if verse_index is None:
                            verse_index = VerseType.from_string(verse_tag, None)
                    if verse_index is None:
                        verse_index = VerseType.from_tag(verse_tag)
                    verse_tag = VerseType.TranslatedTags[verse_index].upper()
                    verse_def = u'%s%s' % (verse_tag, verse[0][u'label'])
                    service_item.add_from_text(unicode(verse[1]), verse_def)
            else:
                # Loop through the verse list and expand the song accordingly.
                for order in song.verse_order.lower().split():
                    if not order:
                        break
                    for verse in verse_list:
                        if verse[0][u'type'][0].lower() == order[0] and \
                            (verse[0][u'label'].lower() == order[1:] or \
                            not order[1:]):
                            if verse_tags_translated:
                                verse_index = VerseType.from_translated_tag(
                                    verse[0][u'type'])
                            else:
                                verse_index = VerseType.from_tag(
                                    verse[0][u'type'])
                            verse_tag = VerseType.TranslatedTags[verse_index]
                            verse_def = u'%s%s' % (verse_tag,
                                verse[0][u'label'])
                            service_item.add_from_text(verse[1], verse_def)
        else:
            verses = song.lyrics.split(u'\n\n')
            for slide in verses:
                service_item.add_from_text(unicode(slide))
        service_item.title = song.title
        author_list = [unicode(author.display_name) for author in song.authors]
        service_item.raw_footer.append(song.title)
        service_item.raw_footer.append(create_separated_list(author_list))
        service_item.raw_footer.append(song.copyright)
        if Settings().value(u'general/ccli number',
            QtCore.QVariant(u'')).toString():
            service_item.raw_footer.append(unicode(
                translate('SongsPlugin.MediaItem', 'CCLI License: ') +
                Settings().value(u'general/ccli number',
                QtCore.QVariant(u'')).toString()))
        service_item.audit = [
            song.title, author_list, song.copyright, unicode(song.ccli_number)
        ]
        service_item.data_string = {u'title': song.search_title,
            u'authors': u', '.join(author_list)}
        service_item.xml_version = self.openLyrics.song_to_xml(song)
        # Add the audio file to the service item.
        if song.media_files:
            service_item.add_capability(ItemCapabilities.HasBackgroundAudio)
            service_item.background_audio = \
                [m.file_name for m in song.media_files]
        return True

    def serviceLoad(self, item):
        """
        Triggered by a song being loaded by the service manager.
        """
        log.debug(u'serviceLoad')
        if self.plugin.status != PluginStatus.Active or not item.data_string:
            return
        if item.data_string[u'title'].find(u'@') == -1:
            # This file seems to be an old one (prior to 1.9.5), which means,
            # that the search title (data_string[u'title']) is probably wrong.
            # We add "@" to search title and hope that we do not add any
            # duplicate. This should work for songs without alternate title.
            search_results = self.plugin.manager.get_all_objects(Song,
                Song.search_title == (re.compile(r'\W+', re.UNICODE).sub(u' ',
                item.data_string[u'title'].strip()) + u'@').strip().lower(),
                Song.search_title.asc())
        else:
            search_results = self.plugin.manager.get_all_objects(Song,
                Song.search_title == item.data_string[u'title'],
                Song.search_title.asc())
        editId = 0
        add_song = True
        temporary = False
        if search_results:
            for song in search_results:
                author_list = item.data_string[u'authors']
                same_authors = True
                for author in song.authors:
                    if author.display_name in author_list:
                        author_list = author_list.replace(author.display_name,
                            u'', 1)
                    else:
                        same_authors = False
                        break
                if same_authors and author_list.strip(u', ') == u'':
                    add_song = False
                    editId = song.id
                    break
                # If there's any backing tracks, copy them over.
                if item.background_audio:
                    self._updateBackgroundAudio(song, item)
        if add_song and self.addSongFromService:
            song = self.openLyrics.xml_to_song(item.xml_version)
            # If there's any backing tracks, copy them over.
            if item.background_audio:
                self._updateBackgroundAudio(song, item)
            editId = song.id
            self.onSearchTextButtonClicked()
        elif add_song and not self.addSongFromService:
            # Make sure we temporary import formatting tags.
            song = self.openLyrics.xml_to_song(item.xml_version, True)
            # If there's any backing tracks, copy them over.
            if item.background_audio:
                self._updateBackgroundAudio(song, item)
            editId = song.id
            temporary = True
        # Update service with correct song id.
        if editId:
            Receiver.send_message(u'service_item_update',
                u'%s:%s:%s' % (editId, item._uuid, temporary))

    def search(self, string, showError):
        """
        Search for some songs
        """
        search_results = self.searchEntire(string)
        return [[song.id, song.title] for song in search_results]



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging

from PyQt4 import QtGui, QtCore, QtWebKit

from openlp.core.lib import ServiceItem, expand_tags, \
    build_lyrics_format_css, build_lyrics_outline_css, Receiver, \
    ItemCapabilities, FormattingTags, ImageSource
from openlp.core.lib.theme import ThemeLevel
from openlp.core.ui import MainDisplay, ScreenList

log = logging.getLogger(__name__)

VERSE = u'The Lord said to {r}Noah{/r}: \n' \
    'There\'s gonna be a {su}floody{/su}, {sb}floody{/sb}\n' \
    'The Lord said to {g}Noah{/g}:\n' \
    'There\'s gonna be a {st}floody{/st}, {it}floody{/it}\n' \
    'Get those children out of the muddy, muddy \n' \
    '{r}C{/r}{b}h{/b}{bl}i{/bl}{y}l{/y}{g}d{/g}{pk}' \
    'r{/pk}{o}e{/o}{pp}n{/pp} of the Lord\n'
VERSE_FOR_LINE_COUNT = u'\n'.join(map(unicode, xrange(50)))
FOOTER = [u'Arky Arky (Unknown)', u'Public Domain', u'CCLI 123456']

class Renderer(object):
    """
    Class to pull all Renderer interactions into one place. The plugins will
    call helper methods to do the rendering but this class will provide
    display defense code.
    """
    log.info(u'Renderer Loaded')

    def __init__(self, image_manager, theme_manager):
        """
        Initialise the renderer.

        ``image_manager``
            A image_manager instance which takes care of e. g. caching and
            resizing images.

        ``theme_manager``
            The theme_manager instance, used to get the current theme details.
        """
        log.debug(u'Initialisation started')
        self.theme_manager = theme_manager
        self.image_manager = image_manager
        self.screens = ScreenList()
        self.theme_level = ThemeLevel.Global
        self.global_theme_name = u''
        self.service_theme_name = u''
        self.item_theme_name = u''
        self.force_page = False
        self.display = MainDisplay(None, self.image_manager, False, self)
        self.display.setup()
        self._theme_dimensions = {}
        self._calculate_default()
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'theme_update_global'), self.set_global_theme)
        self.web = QtWebKit.QWebView()
        self.web.setVisible(False)
        self.web_frame = self.web.page().mainFrame()

    def update_display(self):
        """
        Updates the renderer's information about the current screen.
        """
        log.debug(u'Update Display')
        self._calculate_default()
        if self.display:
            self.display.close()
        self.display = MainDisplay(None, self.image_manager, False, self)
        self.display.setup()
        self._theme_dimensions = {}

    def update_theme(self, theme_name, old_theme_name=None, only_delete=False):
        """
        This method updates the theme in ``_theme_dimensions`` when a theme
        has been edited or renamed.

        ``theme_name``
            The current theme name.

        ``old_theme_name``
            The old theme name. Has only to be passed, when the theme has been
            renamed. Defaults to *None*.

        ``only_delete``
            Only remove the given ``theme_name`` from the ``_theme_dimensions``
            list. This can be used when a theme is permanently deleted.
        """
        if old_theme_name is not None and \
            old_theme_name in self._theme_dimensions:
            del self._theme_dimensions[old_theme_name]
        if theme_name in self._theme_dimensions:
            del self._theme_dimensions[theme_name]
        if not only_delete:
            self._set_theme(theme_name)

    def _set_theme(self, theme_name):
        """
        Helper method to save theme names and theme data.

        ``theme_name``
            The theme name.
        """
        if theme_name not in self._theme_dimensions:
            theme_data = self.theme_manager.getThemeData(theme_name)
            main_rect = self.get_main_rectangle(theme_data)
            footer_rect = self.get_footer_rectangle(theme_data)
            self._theme_dimensions[theme_name] = \
                [theme_data, main_rect, footer_rect]
        else:
            theme_data, main_rect, footer_rect = \
                self._theme_dimensions[theme_name]
        # if No file do not update cache
        if theme_data.background_filename:
            self.image_manager.addImage(theme_data.background_filename,
                ImageSource.Theme,
                QtGui.QColor(theme_data.background_border_color))

    def pre_render(self, override_theme_data=None):
        """
        Set up the theme to be used before rendering an item.

        ``override_theme_data``
            The theme data should be passed, when we want to use our own theme
            data, regardless of the theme level. This should for example be used
            in the theme manager. **Note**, this is **not** to be mixed up with
            the ``set_item_theme`` method.
        """
        # Just assume we use the global theme.
        theme_to_use = self.global_theme_name
        # The theme level is either set to Service or Item. Use the service
        # theme if one is set. We also have to use the service theme, even when
        # the theme level is set to Item, because the item does not necessarily
        # have to have a theme.
        if self.theme_level != ThemeLevel.Global:
            # When the theme level is at Service and we actually have a service
            # theme then use it.
            if self.service_theme_name:
                theme_to_use = self.service_theme_name
        # If we have Item level and have an item theme then use it.
        if self.theme_level == ThemeLevel.Song and self.item_theme_name:
            theme_to_use = self.item_theme_name
        if override_theme_data is None:
            if theme_to_use not in self._theme_dimensions:
                self._set_theme(theme_to_use)
            theme_data, main_rect, footer_rect = \
                self._theme_dimensions[theme_to_use]
        else:
            # Ignore everything and use own theme data.
            theme_data = override_theme_data
            main_rect = self.get_main_rectangle(override_theme_data)
            footer_rect = self.get_footer_rectangle(override_theme_data)
        self._set_text_rectangle(theme_data, main_rect, footer_rect)
        return theme_data, self._rect, self._rect_footer

    def set_theme_level(self, theme_level):
        """
        Sets the theme level.

        ``theme_level``
            The theme level to be used.
        """
        self.theme_level = theme_level

    def set_global_theme(self, global_theme_name):
        """
        Set the global-level theme name.

        ``global_theme_name``
            The global-level theme's name.
        """
        self._set_theme(global_theme_name)
        self.global_theme_name = global_theme_name

    def set_service_theme(self, service_theme_name):
        """
        Set the service-level theme.

        ``service_theme_name``
            The service level theme's name.
        """
        self._set_theme(service_theme_name)
        self.service_theme_name = service_theme_name

    def set_item_theme(self, item_theme_name):
        """
        Set the item-level theme. **Note**, this has to be done for each item we
        are rendering.

        ``item_theme_name``
            The item theme's name.
        """
        self._set_theme(item_theme_name)
        self.item_theme_name = item_theme_name

    def generate_preview(self, theme_data, force_page=False):
        """
        Generate a preview of a theme.

        ``theme_data``
            The theme to generated a preview for.

        ``force_page``
            Flag to tell message lines per page need to be generated.
        """
        log.debug(u'generate preview')
        # save value for use in format_slide
        self.force_page = force_page
        # build a service item to generate preview
        serviceItem = ServiceItem()
        if self.force_page:
            # make big page for theme edit dialog to get line count
            serviceItem.add_from_text(VERSE_FOR_LINE_COUNT)
        else:
            serviceItem.add_from_text(VERSE)
        serviceItem.renderer = self
        serviceItem.raw_footer = FOOTER
        # if No file do not update cache
        if theme_data.background_filename:
            self.image_manager.addImage(theme_data.background_filename,
                ImageSource.Theme,
                QtGui.QColor(theme_data.background_border_color))
        theme_data, main, footer = self.pre_render(theme_data)
        serviceItem.themedata = theme_data
        serviceItem.main = main
        serviceItem.footer = footer
        serviceItem.render(True)
        if not self.force_page:
            self.display.buildHtml(serviceItem)
            raw_html = serviceItem.get_rendered_frame(0)
            self.display.text(raw_html, False)
            preview = self.display.preview()
            return preview
        self.force_page = False

    def format_slide(self, text, item):
        """
        Calculate how much text can fit on a slide.

        ``text``
            The words to go on the slides.

        ``item``
            The :class:`~openlp.core.lib.serviceitem.ServiceItem` item object.
        """
        log.debug(u'format slide')
        # Add line endings after each line of text used for bibles.
        line_end = u'<br>'
        if item.is_capable(ItemCapabilities.NoLineBreaks):
            line_end = u' '
        # Bibles
        if item.is_capable(ItemCapabilities.CanWordSplit):
            pages = self._paginate_slide_words(text.split(u'\n'), line_end)
        # Songs and Custom
        elif item.is_capable(ItemCapabilities.CanSoftBreak):
            pages = []
            if u'[---]' in text:
                # Remove two or more option slide breaks next to each other (causing infinite loop).
                while u'\n[---]\n[---]\n' in text:
                        text = text.replace(u'\n[---]\n[---]\n', u'\n[---]\n')
                while True:
                    slides = text.split(u'\n[---]\n', 2)
                    # If there are (at least) two occurrences of [---] we use
                    # the first two slides (and neglect the last for now).
                    if len(slides) == 3:
                        html_text = expand_tags(u'\n'.join(slides[:2]))
                    # We check both slides to determine if the optional split is
                    # needed (there is only one optional split).
                    else:
                        html_text = expand_tags(u'\n'.join(slides))
                    html_text = html_text.replace(u'\n', u'<br>')
                    if self._text_fits_on_slide(html_text):
                        # The first two optional slides fit (as a whole) on one
                        # slide. Replace the first occurrence of [---].
                        text = text.replace(u'\n[---]', u'', 1)
                    else:
                        # The first optional slide fits, which means we have to
                        # render the first optional slide.
                        text_contains_split = u'[---]' in text
                        if text_contains_split:
                            try:
                                text_to_render, text = \
                                    text.split(u'\n[---]\n', 1)
                            except ValueError:
                                text_to_render = text.split(u'\n[---]\n')[0]
                                text = u''
                            text_to_render, raw_tags, html_tags = \
                                self._get_start_tags(text_to_render)
                            if text:
                                text = raw_tags + text
                        else:
                            text_to_render = text
                            text = u''
                        lines = text_to_render.strip(u'\n').split(u'\n')
                        slides = self._paginate_slide(lines, line_end)
                        if len(slides) > 1 and text:
                            # Add all slides apart from the last one the list.
                            pages.extend(slides[:-1])
                            if  text_contains_split:
                                text = slides[-1] + u'\n[---]\n' + text
                            else:
                                text = slides[-1] + u'\n'+ text
                            text = text.replace(u'<br>', u'\n')
                        else:
                            pages.extend(slides)
                    if u'[---]' not in text:
                        lines = text.strip(u'\n').split(u'\n')
                        pages.extend(self._paginate_slide(lines, line_end))
                        break
            else:
                # Clean up line endings.
                pages = self._paginate_slide(text.split(u'\n'), line_end)
        else:
            pages = self._paginate_slide(text.split(u'\n'), line_end)
        new_pages = []
        for page in pages:
            while page.endswith(u'<br>'):
                page = page[:-4]
            new_pages.append(page)
        return new_pages

    def _calculate_default(self):
        """
        Calculate the default dimensions of the screen.
        """
        screen_size = self.screens.current[u'size']
        self.width = screen_size.width()
        self.height = screen_size.height()
        self.screen_ratio = float(self.height) / float(self.width)
        log.debug(u'_calculate default %s, %f' % (screen_size,
            self.screen_ratio))
        # 90% is start of footer
        self.footer_start = int(self.height * 0.90)

    def get_main_rectangle(self, theme_data):
        """
        Calculates the placement and size of the main rectangle.

        ``theme_data``
            The theme information
        """
        if not theme_data.font_main_override:
            return QtCore.QRect(10, 0, self.width - 20, self.footer_start)
        else:
            return QtCore.QRect(theme_data.font_main_x, theme_data.font_main_y,
                theme_data.font_main_width - 1, theme_data.font_main_height - 1)

    def get_footer_rectangle(self, theme_data):
        """
        Calculates the placement and size of the footer rectangle.

        ``theme_data``
            The theme data.
        """
        if not theme_data.font_footer_override:
            return QtCore.QRect(10, self.footer_start, self.width - 20,
                self.height - self.footer_start)
        else:
            return QtCore.QRect(theme_data.font_footer_x,
                theme_data.font_footer_y, theme_data.font_footer_width - 1,
                theme_data.font_footer_height - 1)

    def _set_text_rectangle(self, theme_data, rect_main, rect_footer):
        """
        Sets the rectangle within which text should be rendered.

        ``theme_data``
            The theme data.

        ``rect_main``
            The main text block.

        ``rect_footer``
            The footer text block.
        """
        log.debug(u'_set_text_rectangle %s , %s' % (rect_main, rect_footer))
        self._rect = rect_main
        self._rect_footer = rect_footer
        self.page_width = self._rect.width()
        self.page_height = self._rect.height()
        if theme_data.font_main_shadow:
            self.page_width -= int(theme_data.font_main_shadow_size)
            self.page_height -= int(theme_data.font_main_shadow_size)
        # For the life of my I don't know why we have to completely kill the
        # QWebView in order for the display to work properly, but we do. See
        # bug #1041366 for an example of what happens if we take this out.
        self.web = None
        self.web = QtWebKit.QWebView()
        self.web.setVisible(False)
        self.web.resize(self.page_width, self.page_height)
        self.web_frame = self.web.page().mainFrame()
        # Adjust width and height to account for shadow. outline done in css.
        html = u"""<!DOCTYPE html><html><head><script>
            function show_text(newtext) {
                var main = document.getElementById('main');
                main.innerHTML = newtext;
                // We need to be sure that the page is loaded, that is why we
                // return the element's height (even though we do not use the
                // returned value).
                return main.offsetHeight;
            }
            </script><style>*{margin: 0; padding: 0; border: 0;}
            #main {position: absolute; top: 0px; %s %s}</style></head><body>
            <div id="main"></div></body></html>""" % \
            (build_lyrics_format_css(theme_data, self.page_width,
            self.page_height), build_lyrics_outline_css(theme_data))
        self.web.setHtml(html)
        self.empty_height = self.web_frame.contentsSize().height()

    def _paginate_slide(self, lines, line_end):
        """
        Figure out how much text can appear on a slide, using the current
        theme settings.
        **Note:** The smallest possible "unit" of text for a slide is one line.
        If the line is too long it will be cut off when displayed.

        ``lines``
            The text to be fitted on the slide split into lines.

        ``line_end``
            The text added after each line. Either ``u' '`` or ``u'<br>``.
        """
        log.debug(u'_paginate_slide - Start')
        formatted = []
        previous_html = u''
        previous_raw = u''
        separator = u'<br>'
        html_lines = map(expand_tags, lines)
        # Text too long so go to next page.
        if not self._text_fits_on_slide(separator.join(html_lines)):
            html_text, previous_raw = self._binary_chop(formatted,
                previous_html, previous_raw, html_lines, lines, separator, u'')
        else:
            previous_raw = separator.join(lines)
        formatted.append(previous_raw)
        log.debug(u'_paginate_slide - End')
        return formatted

    def _paginate_slide_words(self, lines, line_end):
        """
        Figure out how much text can appear on a slide, using the current
        theme settings.
        **Note:** The smallest possible "unit" of text for a slide is one word.
        If one line is too long it will be processed word by word. This is
        sometimes need for **bible** verses.

        ``lines``
            The text to be fitted on the slide split into lines.

        ``line_end``
            The text added after each line. Either ``u' '`` or ``u'<br>``.
            This is needed for **bibles**.
        """
        log.debug(u'_paginate_slide_words - Start')
        formatted = []
        previous_html = u''
        previous_raw = u''
        for line in lines:
            line = line.strip()
            html_line = expand_tags(line)
            # Text too long so go to next page.
            if not self._text_fits_on_slide(previous_html + html_line):
                # Check if there was a verse before the current one and append
                # it, when it fits on the page.
                if previous_html:
                    if self._text_fits_on_slide(previous_html):
                        formatted.append(previous_raw)
                        previous_html = u''
                        previous_raw = u''
                        # Now check if the current verse will fit, if it does
                        # not we have to start to process the verse word by
                        # word.
                        if self._text_fits_on_slide(html_line):
                            previous_html = html_line + line_end
                            previous_raw = line + line_end
                            continue
                # Figure out how many words of the line will fit on screen as
                # the line will not fit as a whole.
                raw_words = self._words_split(line)
                html_words = map(expand_tags, raw_words)
                previous_html, previous_raw = self._binary_chop(
                    formatted, previous_html, previous_raw, html_words,
                    raw_words, u' ', line_end)
            else:
                previous_html += html_line + line_end
                previous_raw += line + line_end
        formatted.append(previous_raw)
        log.debug(u'_paginate_slide_words - End')
        return formatted

    def _get_start_tags(self, raw_text):
        """
        Tests the given text for not closed formatting tags and returns a tuple
        consisting of three unicode strings::

            (u'{st}{r}Text text text{/r}{/st}', u'{st}{r}', u'<strong>
            <span style="-webkit-text-fill-color:red">')

        The first unicode string is the text, with correct closing tags. The
        second unicode string are OpenLP's opening formatting tags and the third
        unicode string the html opening formatting tags.

        ``raw_text``
            The text to test. The text must **not** contain html tags, only
            OpenLP formatting tags are allowed::

                {st}{r}Text text text
        """
        raw_tags = []
        html_tags = []
        for tag in FormattingTags.get_html_tags():
            if tag[u'start tag'] == u'{br}':
                continue
            if raw_text.count(tag[u'start tag']) != \
                raw_text.count(tag[u'end tag']):
                raw_tags.append(
                    (raw_text.find(tag[u'start tag']), tag[u'start tag'],
                    tag[u'end tag']))
                html_tags.append(
                        (raw_text.find(tag[u'start tag']), tag[u'start html']))
        # Sort the lists, so that the tags which were opened first on the first
        # slide (the text we are checking) will be opened first on the next
        # slide as well.
        raw_tags.sort(key=lambda tag: tag[0])
        html_tags.sort(key=lambda tag: tag[0])
        # Create a list with closing tags for the raw_text.
        end_tags = []
        start_tags = []
        for tag in raw_tags:
            start_tags.append(tag[1])
            end_tags.append(tag[2])
        end_tags.reverse()
        # Remove the indexes.
        html_tags = [tag[1] for tag in html_tags]
        return raw_text + u''.join(end_tags),  u''.join(start_tags), \
            u''.join(html_tags)

    def _binary_chop(self, formatted, previous_html, previous_raw, html_list,
        raw_list, separator, line_end):
        """
        This implements the binary chop algorithm for faster rendering. This
        algorithm works line based (line by line) and word based (word by word).
        It is assumed that this method is **only** called, when the lines/words
        to be rendered do **not** fit as a whole.

        ``formatted``
            The list to append any slides.

        ``previous_html``
            The html text which is know to fit on a slide, but is not yet added
            to the list of slides. (unicode string)

        ``previous_raw``
            The raw text (with formatting tags) which is know to fit on a slide,
            but is not yet added to the list of slides. (unicode string)

        ``html_list``
            The elements which do not fit on a slide and needs to be processed
            using the binary chop. The text contains html.

        ``raw_list``
            The elements which do not fit on a slide and needs to be processed
            using the binary chop. The elements can contain formatting tags.

        ``separator``
            The separator for the elements. For lines this is ``u'<br>'`` and
            for words this is ``u' '``.

        ``line_end``
            The text added after each "element line". Either ``u' '`` or
            ``u'<br>``. This is needed for bibles.
        """
        smallest_index = 0
        highest_index = len(html_list) - 1
        index = int(highest_index / 2)
        while True:
            if not self._text_fits_on_slide(
                previous_html + separator.join(html_list[:index + 1]).strip()):
                # We know that it does not fit, so change/calculate the
                # new index and highest_index accordingly.
                highest_index = index
                index = int(index - (index - smallest_index) / 2)
            else:
                smallest_index = index
                index = int(index + (highest_index - index) / 2)
            # We found the number of words which will fit.
            if smallest_index == index or highest_index == index:
                index = smallest_index
                text = previous_raw.rstrip(u'<br>') + \
                    separator.join(raw_list[:index + 1])
                text, raw_tags, html_tags = self._get_start_tags(text)
                formatted.append(text)
                previous_html = u''
                previous_raw = u''
                # Stop here as the theme line count was requested.
                if self.force_page:
                    Receiver.send_message(u'theme_line_count', index + 1)
                    break
            else:
                continue
            # Check if the remaining elements fit on the slide.
            if self._text_fits_on_slide(
                    html_tags + separator.join(html_list[index + 1:]).strip()):
                previous_html = html_tags + separator.join(
                    html_list[index + 1:]).strip() + line_end
                previous_raw = raw_tags + separator.join(
                    raw_list[index + 1:]).strip() + line_end
                break
            else:
                # The remaining elements do not fit, thus reset the indexes,
                # create a new list and continue.
                raw_list = raw_list[index + 1:]
                raw_list[0] = raw_tags + raw_list[0]
                html_list = html_list[index + 1:]
                html_list[0] = html_tags + html_list[0]
                smallest_index = 0
                highest_index = len(html_list) - 1
                index = int(highest_index / 2)
        return previous_html, previous_raw

    def _text_fits_on_slide(self, text):
        """
        Checks if the given ``text`` fits on a slide. If it does ``True`` is
        returned, otherwise ``False``.

        ``text``
            The text to check. It may contain HTML tags.
        """
        self.web_frame.evaluateJavaScript(u'show_text("%s")' %
            text.replace(u'\\', u'\\\\').replace(u'\"', u'\\\"'))
        return self.web_frame.contentsSize().height() <= self.empty_height

    def _words_split(self, line):
        """
        Split the slide up by word so can wrap better
        """
        # this parse we are to be wordy
        line = line.replace(u'\n', u' ')
        return line.split(u' ')



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtGui

from openlp.core.lib import build_icon, translate
from openlp.core.lib.ui import UiStrings, create_button_box, create_button


class Ui_CustomEditDialog(object):
    def setupUi(self, customEditDialog):
        customEditDialog.setObjectName(u'customEditDialog')
        customEditDialog.resize(450, 350)
        customEditDialog.setWindowIcon(build_icon(u':/icon/openlp-logo.svg'))
        self.dialogLayout = QtGui.QVBoxLayout(customEditDialog)
        self.dialogLayout.setObjectName(u'dialogLayout')
        self.titleLayout = QtGui.QHBoxLayout()
        self.titleLayout.setObjectName(u'titleLayout')
        self.titleLabel = QtGui.QLabel(customEditDialog)
        self.titleLabel.setObjectName(u'titleLabel')
        self.titleLayout.addWidget(self.titleLabel)
        self.titleEdit = QtGui.QLineEdit(customEditDialog)
        self.titleLabel.setBuddy(self.titleEdit)
        self.titleEdit.setObjectName(u'titleEdit')
        self.titleLayout.addWidget(self.titleEdit)
        self.dialogLayout.addLayout(self.titleLayout)
        self.centralLayout = QtGui.QHBoxLayout()
        self.centralLayout.setObjectName(u'centralLayout')
        self.slideListView = QtGui.QListWidget(customEditDialog)
        self.slideListView.setAlternatingRowColors(True)
        self.slideListView.setObjectName(u'slideListView')
        self.centralLayout.addWidget(self.slideListView)
        self.buttonLayout = QtGui.QVBoxLayout()
        self.buttonLayout.setObjectName(u'buttonLayout')
        self.addButton = QtGui.QPushButton(customEditDialog)
        self.addButton.setObjectName(u'addButton')
        self.buttonLayout.addWidget(self.addButton)
        self.editButton = QtGui.QPushButton(customEditDialog)
        self.editButton.setEnabled(False)
        self.editButton.setObjectName(u'editButton')
        self.buttonLayout.addWidget(self.editButton)
        self.editAllButton = QtGui.QPushButton(customEditDialog)
        self.editAllButton.setObjectName(u'editAllButton')
        self.buttonLayout.addWidget(self.editAllButton)
        self.deleteButton = create_button(customEditDialog, u'deleteButton',
            role=u'delete', click=customEditDialog.onDeleteButtonClicked)
        self.deleteButton.setEnabled(False)
        self.buttonLayout.addWidget(self.deleteButton)
        self.buttonLayout.addStretch()
        self.upButton = create_button(customEditDialog, u'upButton', role=u'up',
            enabled=False, click=customEditDialog.onUpButtonClicked)
        self.downButton = create_button(customEditDialog, u'downButton',
            role=u'down', enabled=False,
            click=customEditDialog.onDownButtonClicked)
        self.buttonLayout.addWidget(self.upButton)
        self.buttonLayout.addWidget(self.downButton)
        self.centralLayout.addLayout(self.buttonLayout)
        self.dialogLayout.addLayout(self.centralLayout)
        self.bottomFormLayout = QtGui.QFormLayout()
        self.bottomFormLayout.setObjectName(u'bottomFormLayout')
        self.themeLabel = QtGui.QLabel(customEditDialog)
        self.themeLabel.setObjectName(u'themeLabel')
        self.themeComboBox = QtGui.QComboBox(customEditDialog)
        self.themeComboBox.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
        self.themeComboBox.setObjectName(u'themeComboBox')
        self.themeLabel.setBuddy(self.themeComboBox)
        self.bottomFormLayout.addRow(self.themeLabel, self.themeComboBox)
        self.creditLabel = QtGui.QLabel(customEditDialog)
        self.creditLabel.setObjectName(u'creditLabel')
        self.creditEdit = QtGui.QLineEdit(customEditDialog)
        self.creditEdit.setObjectName(u'creditEdit')
        self.creditLabel.setBuddy(self.creditEdit)
        self.bottomFormLayout.addRow(self.creditLabel, self.creditEdit)
        self.dialogLayout.addLayout(self.bottomFormLayout)
        self.previewButton = QtGui.QPushButton()
        self.buttonBox = create_button_box(customEditDialog, u'buttonBox',
            [u'cancel', u'save'], [self.previewButton])
        self.dialogLayout.addWidget(self.buttonBox)
        self.retranslateUi(customEditDialog)

    def retranslateUi(self, customEditDialog):
        customEditDialog.setWindowTitle(
            translate('CustomPlugin.EditCustomForm', 'Edit Custom Slides'))
        self.titleLabel.setText(
            translate('CustomPlugin.EditCustomForm', '&Title:'))
        self.addButton.setText(UiStrings().Add)
        self.addButton.setToolTip(
            translate('CustomPlugin.EditCustomForm', 'Add a new slide at '
            'bottom.'))
        self.editButton.setText(UiStrings().Edit)
        self.editButton.setToolTip(
            translate('CustomPlugin.EditCustomForm', 'Edit the selected '
            'slide.'))
        self.editAllButton.setText(
            translate('CustomPlugin.EditCustomForm', 'Ed&it All'))
        self.editAllButton.setToolTip(
            translate('CustomPlugin.EditCustomForm', 'Edit all the slides at '
            'once.'))
        self.themeLabel.setText(
            translate('CustomPlugin.EditCustomForm', 'The&me:'))
        self.creditLabel.setText(
            translate('CustomPlugin.EditCustomForm', '&Credits:'))
        self.previewButton.setText(UiStrings().SaveAndPreview)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
The :mod:`ewimport` module provides the functionality for importing
EasyWorship song databases into the current installation database.
"""

import os
import struct
import re

from openlp.core.lib import translate
from openlp.plugins.songs.lib import VerseType
from openlp.plugins.songs.lib import retrieve_windows_encoding, strip_rtf
from songimport import SongImport

RTF_STRIPPING_REGEX = re.compile(r'\{\\tx[^}]*\}')
# regex: at least two newlines, can have spaces between them
SLIDE_BREAK_REGEX = re.compile(r'\n *?\n[\n ]*')
NUMBER_REGEX = re.compile(r'[0-9]+')
NOTE_REGEX = re.compile(r'\(.*?\)')


class FieldDescEntry:
    def __init__(self, name, type, size):
        self.name = name
        self.type = type
        self.size = size


class EasyWorshipSongImport(SongImport):
    """
    The :class:`EasyWorshipSongImport` class provides OpenLP with the
    ability to import EasyWorship song files.
    """
    def __init__(self, manager, **kwargs):
        SongImport.__init__(self, manager, **kwargs)

    def doImport(self):
        # Open the DB and MB files if they exist
        import_source_mb = self.importSource.replace('.DB', '.MB').replace('.db', '.mb')
        if not os.path.isfile(self.importSource):
            self.logError(self.importSource, translate(
                'SongsPlugin.EasyWorshipSongImport',
                'This file does not exist.'))
            return
        if not os.path.isfile(import_source_mb):
            self.logError(self.importSource, translate(
                'SongsPlugin.EasyWorshipSongImport', 
                'Could not find the "Songs.MB" file. It must be in the same '
                'folder as the "Songs.DB" file.'))
            return
        db_size = os.path.getsize(self.importSource)
        if db_size < 0x800:
            self.logError(self.importSource, translate(
                'SongsPlugin.EasyWorshipSongImport',
                'This file is not a valid EasyWorship database.'))
            return
        db_file = open(self.importSource, 'rb')
        self.memoFile = open(import_source_mb, 'rb')
        # Don't accept files that are clearly not paradox files
        record_size, header_size, block_size, first_block, num_fields \
            = struct.unpack('<hhxb8xh17xh', db_file.read(35))
        if header_size != 0x800 or block_size < 1 or block_size > 4:
            db_file.close()
            self.memoFile.close()
            self.logError(self.importSource, translate(
                'SongsPlugin.EasyWorshipSongImport',
                'This file is not a valid EasyWorship database.'))
            return
        # Take a stab at how text is encoded
        self.encoding = u'cp1252'
        db_file.seek(106)
        code_page, = struct.unpack('<h', db_file.read(2))
        if code_page == 852:
            self.encoding = u'cp1250'
        # The following codepage to actual encoding mappings have not been
        # observed, but merely guessed. Actual example files are needed.
        elif code_page == 737:
            self.encoding = u'cp1253'
        elif code_page == 775:
            self.encoding = u'cp1257'
        elif code_page == 855:
            self.encoding = u'cp1251'
        elif code_page == 857:
            self.encoding = u'cp1254'
        elif code_page == 866:
            self.encoding = u'cp1251'
        elif code_page == 869:
            self.encoding = u'cp1253'
        elif code_page == 862:
            self.encoding = u'cp1255'
        elif code_page == 874:
            self.encoding = u'cp874'
        self.encoding = retrieve_windows_encoding(self.encoding)
        if not self.encoding:
            return
        # Read the field description information
        db_file.seek(120)
        field_info = db_file.read(num_fields * 2)
        db_file.seek(4 + (num_fields * 4) + 261, os.SEEK_CUR)
        field_names = db_file.read(header_size - db_file.tell()).split('\0',
            num_fields)
        field_names.pop()
        field_descs = []
        for i, field_name in enumerate(field_names):
            field_type, field_size = struct.unpack_from('BB',
                field_info, i * 2)
            field_descs.append(FieldDescEntry(field_name, field_type,
                field_size))
        self.setRecordStruct(field_descs)
        # Pick out the field description indexes we will need
        try:
            success = True
            fi_title = self.findField(u'Title')
            fi_author = self.findField(u'Author')
            fi_copy = self.findField(u'Copyright')
            fi_admin = self.findField(u'Administrator')
            fi_words = self.findField(u'Words')
            fi_ccli = self.findField(u'Song Number')
        except IndexError:
            # This is the wrong table
            success = False
        # There does not appear to be a _reliable_ way of getting the number
        # of songs/records, so loop through the file blocks and total the
        # number of records. Store the information in a list so we dont have
        # to do all this again.
        cur_block = first_block
        total_count = 0
        block_list = []
        while cur_block != 0 and success:
            cur_block_pos = header_size + ((cur_block - 1) * 1024 * block_size)
            db_file.seek(cur_block_pos)
            cur_block, rec_count = struct.unpack('<h2xh', db_file.read(6))
            rec_count = (rec_count + record_size) / record_size
            block_list.append((cur_block_pos, rec_count))
            total_count += rec_count
        self.importWizard.progressBar.setMaximum(total_count)
        for block in block_list:
            cur_block_pos, rec_count = block
            db_file.seek(cur_block_pos + 6)
            # Loop through each record within the current block
            for i in range(rec_count):
                if self.stopImportFlag:
                    break
                raw_record = db_file.read(record_size)
                self.fields = self.recordStruct.unpack(raw_record)
                self.setDefaults()
                self.title = self.getField(fi_title)
                # Get remaining fields.
                copy = self.getField(fi_copy)
                admin = self.getField(fi_admin)
                ccli = self.getField(fi_ccli)
                authors = self.getField(fi_author)
                words = self.getField(fi_words)
                # Set the SongImport object members.
                if copy:
                    self.copyright = copy
                if admin:
                    if copy:
                        self.copyright += u', '
                    self.copyright += \
                        unicode(translate('SongsPlugin.EasyWorshipSongImport',
                            'Administered by %s')) % admin
                if ccli:
                    self.ccliNumber = ccli
                if authors:
                    # Split up the authors
                    author_list = authors.split(u'/')
                    if len(author_list) < 2:
                        author_list = authors.split(u';')
                    if len(author_list) < 2:
                        author_list = authors.split(u',')
                    for author_name in author_list:
                        self.addAuthor(author_name.strip())
                if words:
                    # Format the lyrics
                    result = None
                    try:
                        result = strip_rtf(words, self.encoding)
                    except UnicodeDecodeError:
                        # The unicode chars in the rtf was not escaped in the expected manner.
                        self.logError(self.title, unicode(translate(
                            'SongsPlugin.EasyWorshipSongImport',
                            'Unexpected data formatting.')))
                        continue
                    if result is None:
                        return
                    words, self.encoding = result
                    verse_type = VerseType.Tags[VerseType.Verse]
                    for verse in SLIDE_BREAK_REGEX.split(words):
                        verse = verse.strip()
                        if not verse:
                            continue
                        verse_split = verse.split(u'\n', 1)
                        first_line_is_tag = False
                        # EW tags: verse, chorus, pre-chorus, bridge, tag,
                        # intro, ending, slide
                        for type in VerseType.Names+[u'tag', u'slide']:
                            type = type.lower()
                            ew_tag = verse_split[0].strip().lower()
                            if ew_tag.startswith(type):
                                verse_type = type[0]
                                if type == u'tag' or type == u'slide':
                                    verse_type = VerseType.Tags[VerseType.Other]
                                first_line_is_tag = True
                                number_found = False
                                # check if tag is followed by number and/or note
                                if len(ew_tag) > len(type):
                                    match = NUMBER_REGEX.search(ew_tag)
                                    if match:
                                        number = match.group()
                                        verse_type += number
                                        number_found = True
                                    match = NOTE_REGEX.search(ew_tag)
                                    if match:
                                        self.comments += ew_tag + u'\n'
                                if not number_found:
                                    verse_type += u'1'
                                break
                        self.addVerse(
                            verse_split[-1].strip() \
                                if first_line_is_tag else verse,
                            verse_type)
                if len(self.comments) > 5:
                    self.comments += unicode(
                        translate('SongsPlugin.EasyWorshipSongImport',
                        '\n[above are Song Tags with notes imported from \
                        EasyWorship]'))
                if self.stopImportFlag:
                    break
                if not self.finish():
                    self.logError(self.importSource)
        db_file.close()
        self.memoFile.close()

    def findField(self, field_name):
        return [i for i, x in enumerate(self.fieldDescs)
            if x.name == field_name][0]

    def setRecordStruct(self, field_descs):
        # Begin with empty field struct list
        fsl = ['>']
        for field_desc in field_descs:
            if field_desc.type == 1:
                # string
                fsl.append('%ds' % field_desc.size)
            elif field_desc.type == 3:
                # 16-bit int
                fsl.append('H')
            elif field_desc.type == 4:
                # 32-bit int
                fsl.append('I')
            elif field_desc.type == 9:
                # Logical
                fsl.append('B')
            elif field_desc.type == 0x0c:
                # Memo
                fsl.append('%ds' % field_desc.size)
            elif field_desc.type == 0x0d:
                # Blob
                fsl.append('%ds' % field_desc.size)
            elif field_desc.type == 0x15:
                # Timestamp
                fsl.append('Q')
            else:
                fsl.append('%ds' % field_desc.size)
        self.recordStruct = struct.Struct(''.join(fsl))
        self.fieldDescs = field_descs

    def getField(self, field_desc_index):
        field = self.fields[field_desc_index]
        field_desc = self.fieldDescs[field_desc_index]
        # Return None in case of 'blank' entries
        if isinstance(field, str):
            if not field.rstrip('\0'):
                return None
        elif field == 0:
            return None
        # Format the field depending on the field type
        if field_desc.type == 1:
            # string
            return field.rstrip('\0').decode(self.encoding)
        elif field_desc.type == 3:
            # 16-bit int
            return field ^ 0x8000
        elif field_desc.type == 4:
            # 32-bit int
            return field ^ 0x80000000
        elif field_desc.type == 9:
            # Logical
            return (field ^ 0x80 == 1)
        elif field_desc.type == 0x0c or field_desc.type == 0x0d:
            # Memo or Blob
            block_start, blob_size = \
                struct.unpack_from('<II', field, len(field)-10)
            sub_block = block_start & 0xff
            block_start &= ~0xff
            self.memoFile.seek(block_start)
            memo_block_type, = struct.unpack('b', self.memoFile.read(1))
            if memo_block_type == 2:
                self.memoFile.seek(8, os.SEEK_CUR)
            elif memo_block_type == 3:
                if sub_block > 63:
                    return u''
                self.memoFile.seek(11 + (5 * sub_block), os.SEEK_CUR)
                sub_block_start, = struct.unpack('B', self.memoFile.read(1))
                self.memoFile.seek(block_start + (sub_block_start * 16))
            else:
                return u''
            return self.memoFile.read(blob_size)
        else:
            return 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Edwin Lunando, Joshua Miller, Stevan Pettit,  #
# Andreas Preikschat, Mattias Põldaru, Christian Richter, Philip Ridout,      #
# Simon Scudder, Jeffrey Smith, Maikel Stuivenberg, Martin Thompson, Jon      #
# Tibble, Dave Warnock, Frode Woldsund                                        #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

"""
This script is used to maintain the translation files in OpenLP. It downloads
the latest translation files from the Transifex translation server, updates the
local translation files from both the source code and the files from Transifex,
and can also generate the compiled translation files.

Create New Language
-------------------

To create a new language, simply run this script with the ``-c`` command line
option::

    @:~$ ./translation_utils.py -c

Update Translation Files
------------------------

The best way to update the translations is to download the files from Transifex,
and then update the local files using both the downloaded files and the source.
This is done easily via the ``-d``, ``-p`` and ``-u`` options::

    @:~$ ./translation_utils.py -dpu

"""
import os
import urllib2
import re
from shutil import copy
from getpass import getpass
import base64
import json
import webbrowser

from optparse import OptionParser
from PyQt4 import QtCore
from BeautifulSoup import BeautifulSoup

SERVER_URL = u'http://www.transifex.net/api/2/project/openlp/'
IGNORED_PATHS = [u'scripts']
IGNORED_FILES = [u'setup.py']

verbose_mode = False
quiet_mode = False
username = ''
password = ''

class Command(object):
    """
    Provide an enumeration of commands.
    """
    Download = 1
    Create = 2
    Prepare = 3
    Update = 4
    Generate = 5

class CommandStack(object):
    """
    This class provides an iterable stack.
    """
    def __init__(self):
        self.current_index = 0
        self.data = []

    def __len__(self):
        return len(self.data)

    def __getitem__(self, index):
        if not index in self.data:
            return None
        elif self.data[index].get(u'arguments'):
            return self.data[index][u'command'], self.data[index][u'arguments']
        else:
            return self.data[index][u'command']

    def __iter__(self):
        return self

    def next(self):
        if self.current_index == len(self.data):
            raise StopIteration
        else:
            current_item = self.data[self.current_index][u'command']
            self.current_index += 1
            return current_item

    def append(self, command, **kwargs):
        data = {u'command': command}
        if u'arguments' in kwargs:
            data[u'arguments'] = kwargs[u'arguments']
        self.data.append(data)

    def reset(self):
        self.current_index = 0

    def arguments(self):
        if self.data[self.current_index - 1].get(u'arguments'):
            return self.data[self.current_index - 1][u'arguments']
        else:
            return []

    def __repr__(self):
        results = []
        for item in self.data:
            if item.get(u'arguments'):
                results.append(str((item[u'command'], item[u'arguments'])))
            else:
                results.append(str((item[u'command'], )))
        return u'[%s]' % u', '.join(results)

def print_quiet(text, linefeed=True):
    """
    This method checks to see if we are in quiet mode, and if not prints
    ``text`` out.

    ``text``
        The text to print.
    """
    global quiet_mode
    if not quiet_mode:
        if linefeed:
            print text
        else:
            print text,

def print_verbose(text):
    """
    This method checks to see if we are in verbose mode, and if so prints
    ``text`` out.

    ``text``
        The text to print.
    """
    global verbose_mode, quiet_mode
    if not quiet_mode and verbose_mode:
        print u'    %s' % text

def run(command):
    """
    This method runs an external application.

    ``command``
        The command to run.
    """
    print_verbose(command)
    process = QtCore.QProcess()
    process.start(command)
    while (process.waitForReadyRead()):
        print_verbose(u'ReadyRead: %s' % QtCore.QString(process.readAll()))
    print_verbose(u'Error(s):\n%s' % process.readAllStandardError())
    print_verbose(u'Output:\n%s' % process.readAllStandardOutput())

def download_translations():
    """
    This method downloads the translation files from the Pootle server.

    **Note:** URLs and headers need to remain strings, not unicode.
    """
    global username, password
    print_quiet(u'Download translation files from Transifex')
    if not username:
        username = raw_input(u'   Transifex username: ')
    if not password:
        password = getpass(u'   Transifex password: ')
    # First get the list of languages
    url = SERVER_URL + 'resource/ents/'
    base64string = base64.encodestring(
        '%s:%s' % (username, password))[:-1]
    auth_header =  'Basic %s' % base64string
    request = urllib2.Request(url + '?details')
    request.add_header('Authorization', auth_header)
    print_verbose(u'Downloading list of languages from: %s' % url)
    try:
        json_response = urllib2.urlopen(request)
    except urllib2.HTTPError:
        print_quiet(u'Username or password incorrect.')
        return False
    json_dict = json.loads(json_response.read())
    languages = [lang[u'code'] for lang in json_dict[u'available_languages']]
    for language in languages:
        lang_url = url + 'translation/%s/?file' % language
        request = urllib2.Request(lang_url)
        request.add_header('Authorization', auth_header)
        filename = os.path.join(os.path.abspath(u'..'), u'resources', u'i18n',
            language + u'.ts')
        print_verbose(u'Get Translation File: %s' % filename)
        response = urllib2.urlopen(request)
        fd = open(filename, u'w')
        fd.write(response.read())
        fd.close()
    print_quiet(u'   Done.')
    return True

def prepare_project():
    """
    This method creates the project file needed to update the translation files
    and compile them into .qm files.
    """
    print_quiet(u'Generating the openlp.pro file')
    lines = []
    start_dir = os.path.abspath(u'..')
    start_dir = start_dir + os.sep
    print_verbose(u'Starting directory: %s' % start_dir)
    for root, dirs, files in os.walk(start_dir):
        for file in files:
            path = root.replace(start_dir, u'').replace(u'\\', u'/') #.replace(u'..', u'.')
            if file.startswith(u'hook-') or file.startswith(u'test_'):
               continue
            ignore = False
            for ignored_path in IGNORED_PATHS:
                if path.startswith(ignored_path):
                    ignore = True
                    break
            if ignore:
                continue
            ignore = False
            for ignored_file in IGNORED_FILES:
                if file == ignored_file:
                    ignore = True
                    break
            if ignore:
                continue
            if file.endswith(u'.py') or file.endswith(u'.pyw'):
                if path:
                    line = u'%s/%s' % (path, file)
                else:
                    line = file
                print_verbose(u'Parsing "%s"' % line)
                lines.append(u'SOURCES      += %s' % line)
            elif file.endswith(u'.ts'):
                line = u'%s/%s' % (path, file)
                print_verbose(u'Parsing "%s"' % line)
                lines.append(u'TRANSLATIONS += %s' % line)
    lines.sort()
    file = open(os.path.join(start_dir, u'openlp.pro'), u'w')
    file.write(u'\n'.join(lines).encode('utf8'))
    file.close()
    print_quiet(u'   Done.')

def update_translations():
    print_quiet(u'Update the translation files')
    if not os.path.exists(os.path.join(os.path.abspath(u'..'), u'openlp.pro')):
        print u'You have not generated a project file yet, please run this ' + \
            u'script with the -p option.'
        return
    else:
        os.chdir(os.path.abspath(u'..'))
        run(u'pylupdate4 -verbose -noobsolete openlp.pro')
        os.chdir(os.path.abspath(u'scripts'))

def generate_binaries():
    print_quiet(u'Generate the related *.qm files')
    if not os.path.exists(os.path.join(os.path.abspath(u'..'), u'openlp.pro')):
        print u'You have not generated a project file yet, please run this ' + \
            u'script with the -p option. It is also recommended that you ' + \
            u'this script with the -u option to update the translation ' + \
            u'files as well.'
        return
    else:
        os.chdir(os.path.abspath(u'..'))
        run(u'lrelease openlp.pro')
        print_quiet(u'   Done.')


def create_translation():
    """
    This method opens a browser to the OpenLP project page at Transifex so
    that the user can request a new language.
    """
    print_quiet(u'Please request a new language at the OpenLP project on '
        'Transifex.')
    webbrowser.open('https://www.transifex.net/projects/p/openlp/'
        'resource/ents/')
    print_quiet(u'Opening browser to OpenLP project...')

def process_stack(command_stack):
    """
    This method looks at the commands in the command stack, and processes them
    in the order they are in the stack.

    ``command_stack``
        The command stack to process.
    """
    if command_stack:
        print_quiet(u'Processing %d commands...' % len(command_stack))
        for command in command_stack:
            print_quiet(u'%d.' % (command_stack.current_index), False)
            if command == Command.Download:
                if not download_translations():
                    return
            elif command == Command.Prepare:
                prepare_project()
            elif command == Command.Update:
                update_translations()
            elif command == Command.Generate:
                generate_binaries()
            elif command == Command.Create:
                create_translation()
        print_quiet(u'Finished processing commands.')
    else:
        print_quiet(u'No commands to process.')

def main():
    global verbose_mode, quiet_mode, username, password
    # Set up command line options.
    usage = u'%prog [options]\nOptions are parsed in the order they are ' + \
        u'listed below. If no options are given, "-dpug" will be used.\n\n' + \
        u'This script is used to manage OpenLP\'s translation files.'
    parser = OptionParser(usage=usage)
    parser.add_option('-U', '--username', dest='username', metavar='USERNAME',
        help='Transifex username, used for authentication')
    parser.add_option('-P', '--password', dest='password', metavar='PASSWORD',
        help='Transifex password, used for authentication')
    parser.add_option('-d', '--download-ts', dest='download',
        action='store_true', help='download language files from Transifex')
    parser.add_option('-c', '--create', dest='create', action='store_true',
        help='go to Transifex to request a new translation file')
    parser.add_option('-p', '--prepare', dest='prepare', action='store_true',
        help='generate a project file, used to update the translations')
    parser.add_option('-u', '--update', action='store_true', dest='update',
        help='update translation files (needs a project file)')
    parser.add_option('-g', '--generate', dest='generate', action='store_true',
        help='compile .ts files into .qm files')
    parser.add_option('-v', '--verbose', dest='verbose', action='store_true',
        help='show extra information while processing translations')
    parser.add_option('-q', '--quiet', dest='quiet', action='store_true',
        help='suppress all output other than errors')
    (options, args) = parser.parse_args()
    # Create and populate the command stack
    command_stack = CommandStack()
    if options.download:
        command_stack.append(Command.Download)
    if options.create:
        command_stack.append(Command.Create, arguments=[options.create])
    if options.prepare:
        command_stack.append(Command.Prepare)
    if options.update:
        command_stack.append(Command.Update)
    if options.generate:
        command_stack.append(Command.Generate)
    verbose_mode = options.verbose
    quiet_mode = options.quiet
    if options.username:
        username = options.username
    if options.password:
        password = options.password
    if not command_stack:
        command_stack.append(Command.Download)
        command_stack.append(Command.Prepare)
        command_stack.append(Command.Update)
        command_stack.append(Command.Generate)
    # Process the commands
    process_stack(command_stack)

if __name__ == u'__main__':
    if os.path.split(os.path.abspath(u'.'))[1] != u'scripts':
        print u'You need to run this script from the scripts directory.'
    else:
        main()



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
The :mod:`songbeamerimport` module provides the functionality for importing
 SongBeamer songs into the OpenLP database.
"""
import chardet
import codecs
import logging
import os
import re

from openlp.plugins.songs.lib import VerseType
from openlp.plugins.songs.lib.songimport import SongImport

log = logging.getLogger(__name__)

class SongBeamerTypes(object):
    MarkTypes = {
        u'Refrain': VerseType.Tags[VerseType.Chorus],
        u'Chorus': VerseType.Tags[VerseType.Chorus],
        u'Vers': VerseType.Tags[VerseType.Verse],
        u'Verse': VerseType.Tags[VerseType.Verse],
        u'Strophe': VerseType.Tags[VerseType.Verse],
        u'Intro': VerseType.Tags[VerseType.Intro],
        u'Coda': VerseType.Tags[VerseType.Ending],
        u'Ending': VerseType.Tags[VerseType.Ending],
        u'Bridge': VerseType.Tags[VerseType.Bridge],
        u'Interlude': VerseType.Tags[VerseType.Bridge],
        u'Zwischenspiel': VerseType.Tags[VerseType.Bridge],
        u'Pre-Chorus': VerseType.Tags[VerseType.PreChorus],
        u'Pre-Refrain': VerseType.Tags[VerseType.PreChorus],
        u'Pre-Bridge': VerseType.Tags[VerseType.Other],
        u'Pre-Coda': VerseType.Tags[VerseType.Other],
        u'Unbekannt': VerseType.Tags[VerseType.Other],
        u'Unknown': VerseType.Tags[VerseType.Other],
        u'Unbenannt': VerseType.Tags[VerseType.Other]
    }


class SongBeamerImport(SongImport):
    """
    Import Song Beamer files(s)
    Song Beamer file format is text based
    in the beginning are one or more control tags written
    """
    HTML_TAG_PAIRS = [
        (re.compile(u'<b>'), u'{st}'),
        (re.compile(u'</b>'), u'{/st}'),
        (re.compile(u'<i>'), u'{it}'),
        (re.compile(u'</i>'), u'{/it}'),
        (re.compile(u'<u>'), u'{u}'),
        (re.compile(u'</u>'), u'{/u}'),
        (re.compile(u'<p>'), u'{p}'),
        (re.compile(u'</p>'), u'{/p}'),
        (re.compile(u'<super>'), u'{su}'),
        (re.compile(u'</super>'), u'{/su}'),
        (re.compile(u'<sub>'), u'{sb}'),
        (re.compile(u'</sub>'), u'{/sb}'),
        (re.compile(u'<br.*?>'), u'{br}'),
        (re.compile(u'<[/]?wordwrap>'), u''),
        (re.compile(u'<[/]?strike>'), u''),
        (re.compile(u'<[/]?h.*?>'), u''),
        (re.compile(u'<[/]?s.*?>'), u''),
        (re.compile(u'<[/]?linespacing.*?>'), u''),
        (re.compile(u'<[/]?c.*?>'), u''),
        (re.compile(u'<align.*?>'), u''),
        (re.compile(u'<valign.*?>'), u'')
    ]

    def __init__(self, manager, **kwargs):
        """
        Initialise the Song Beamer importer.
        """
        SongImport.__init__(self, manager, **kwargs)

    def doImport(self):
        """
        Receive a single file or a list of files to import.
        """
        self.importWizard.progressBar.setMaximum(len(self.importSource))
        if not isinstance(self.importSource, list):
            return
        for file in self.importSource:
            # TODO: check that it is a valid SongBeamer file
            if self.stopImportFlag:
                return
            self.setDefaults()
            self.currentVerse = u''
            self.currentVerseType = VerseType.Tags[VerseType.Verse]
            read_verses = False
            file_name = os.path.split(file)[1]
            if os.path.isfile(file):
                detect_file = open(file, u'r')
                details = chardet.detect(detect_file.read())
                detect_file.close()
                infile = codecs.open(file, u'r', details['encoding'])
                song_data = infile.readlines()
                infile.close()
            else:
                continue
            self.title = file_name.split('.sng')[0]
            read_verses = False
            for line in song_data:
                # Just make sure that the line is of the type 'Unicode'.
                line = unicode(line).strip()
                if line.startswith(u'#') and not read_verses:
                    self.parseTags(line)
                elif line.startswith(u'---'):
                    if self.currentVerse:
                        self.replaceHtmlTags()
                        self.addVerse(self.currentVerse,
                            self.currentVerseType)
                        self.currentVerse = u''
                        self.currentVerseType = VerseType.Tags[VerseType.Verse]
                    read_verses = True
                    verse_start = True
                elif read_verses:
                    if verse_start:
                        verse_start = False
                        if not self.checkVerseMarks(line):
                            self.currentVerse = line + u'\n'
                    else:
                        self.currentVerse += line + u'\n'
            if self.currentVerse:
                self.replaceHtmlTags()
                self.addVerse(self.currentVerse, self.currentVerseType)
            if not self.finish():
                self.logError(file)

    def replaceHtmlTags(self):
        """
        This can be called to replace SongBeamer's specific (html) tags with
        OpenLP's specific (html) tags.
        """
        for pair in SongBeamerImport.HTML_TAG_PAIRS:
            self.currentVerse = pair[0].sub(pair[1], self.currentVerse)

    def parseTags(self, line):
        """
        Parses a meta data line.

        ``line``
            The line in the file. It should consist of a tag and a value
            for this tag (unicode)::

                u'#Title=Nearer my God to Thee'
        """
        tag_val = line.split(u'=', 1)
        if len(tag_val) == 1:
            return
        if not tag_val[0] or not tag_val[1]:
            return
        if tag_val[0] == u'#(c)':
            self.addCopyright(tag_val[1])
        elif tag_val[0] == u'#AddCopyrightInfo':
            pass
        elif tag_val[0] == u'#Author':
            self.parseAuthor(tag_val[1])
        elif tag_val[0] == u'#BackgroundImage':
            pass
        elif tag_val[0] == u'#Bible':
            pass
        elif tag_val[0] == u'#Categories':
            self.topics = tag_val[1].split(',')
        elif tag_val[0] == u'#CCLI':
            self.ccliNumber = tag_val[1]
        elif tag_val[0] == u'#Chords':
            pass
        elif tag_val[0] == u'#ChurchSongID':
            pass
        elif tag_val[0] == u'#ColorChords':
            pass
        elif tag_val[0] == u'#Comments':
            self.comments = tag_val[1]
        elif tag_val[0] == u'#Editor':
            pass
        elif tag_val[0] == u'#Font':
            pass
        elif tag_val[0] == u'#FontLang2':
            pass
        elif tag_val[0] == u'#FontSize':
            pass
        elif tag_val[0] == u'#Format':
            pass
        elif tag_val[0] == u'#Format_PreLine':
            pass
        elif tag_val[0] == u'#Format_PrePage':
            pass
        elif tag_val[0] == u'#ID':
            pass
        elif tag_val[0] == u'#Key':
            pass
        elif tag_val[0] == u'#Keywords':
            pass
        elif tag_val[0] == u'#LangCount':
            pass
        elif tag_val[0] == u'#Melody':
            self.parseAuthor(tag_val[1])
        elif tag_val[0] == u'#NatCopyright':
            pass
        elif tag_val[0] == u'#OTitle':
            pass
        elif tag_val[0] == u'#OutlineColor':
            pass
        elif tag_val[0] == u'#OutlinedFont':
            pass
        elif tag_val[0] == u'#QuickFind':
            pass
        elif tag_val[0] == u'#Rights':
            song_book_pub = tag_val[1]
        elif tag_val[0] == u'#Songbook' or tag_val[0] == u'#SongBook':
            book_data = tag_val[1].split(u'/')
            self.songBookName = book_data[0].strip()
            if len(book_data) == 2:
                number = book_data[1].strip()
                self.songNumber = number if number.isdigit() else u''
        elif tag_val[0] == u'#Speed':
            pass
        elif tag_val[0] == u'Tempo':
            pass
        elif tag_val[0] == u'#TextAlign':
            pass
        elif tag_val[0] == u'#Title':
            self.title = unicode(tag_val[1])
        elif tag_val[0] == u'#TitleAlign':
            pass
        elif tag_val[0] == u'#TitleFontSize':
            pass
        elif tag_val[0] == u'#TitleLang2':
            pass
        elif tag_val[0] == u'#TitleLang3':
            pass
        elif tag_val[0] == u'#TitleLang4':
            pass
        elif tag_val[0] == u'#Translation':
            pass
        elif tag_val[0] == u'#Transpose':
            pass
        elif tag_val[0] == u'#TransposeAccidental':
            pass
        elif tag_val[0] == u'#Version':
            pass
        elif tag_val[0] == u'#VerseOrder':
            # TODO: add the verse order.
            pass

    def checkVerseMarks(self, line):
        """
        Check and add the verse's MarkType. Returns ``True`` if the given line
        contains a correct verse mark otherwise ``False``.

        ``line``
            The line to check for marks (unicode).
        """
        marks = line.split(u' ')
        if len(marks) <= 2 and marks[0] in SongBeamerTypes.MarkTypes:
            self.currentVerseType = SongBeamerTypes.MarkTypes[marks[0]]
            if len(marks) == 2:
                # If we have a digit, we append it to current_verse_type.
                if marks[1].isdigit():
                    self.currentVerseType += marks[1]
            return True
        return False
